window.Gleam.initSearch([{"doc":"lustre","title":"lustre","content":"&lt;h1 align=&quot;center&quot;&gt;Lustre&lt;/h1&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  ✨ &lt;strong&gt;Make your frontend shine&lt;/strong&gt; ✨\n&lt;/div&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  A framework for building Web apps in Gleam!\n&lt;/div&gt;\n\n&lt;br /&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;a href=&quot;https://hex.pm/packages/lustre&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/hexpm/v/lustre&quot;\n      alt=&quot;Available on Hex&quot; /&gt;\n  &lt;/a&gt;\n  &lt;a href=&quot;https://hexdocs.pm/lustre&quot;&gt;\n    &lt;img src=&quot;https://img.shields.io/badge/hex-docs-ffaff3&quot;\n      alt=&quot;Documentation&quot; /&gt;\n  &lt;/a&gt;\n&lt;/div&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;h3&gt;\n    &lt;!--\n    &lt;a href=&quot;https://lustre.build&quot;&gt;\n      Website\n    &lt;/a&gt;\n    &lt;span&gt; | &lt;/span&gt;\n    --&gt;\n    &lt;a href=&quot;https://hexdocs.pm/lustre/guide/01-quickstart.html&quot;&gt;\n      Quickstart\n    &lt;/a&gt;\n    &lt;span&gt; | &lt;/span&gt;\n    &lt;a href=&quot;https://hexdocs.pm/lustre&quot;&gt;\n      Reference\n    &lt;/a&gt;\n    &lt;span&gt; | &lt;/span&gt;\n    &lt;a href=&quot;https://discord.gg/Fm8Pwmy&quot;&gt;\n      Discord\n    &lt;/a&gt;\n  &lt;/h3&gt;\n&lt;/div&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  &lt;sub&gt;Built with ❤︎ by\n  &lt;a href=&quot;https://twitter.com/hayleighdotdev&quot;&gt;Hayleigh Thompson&lt;/a&gt; and\n  &lt;a href=&quot;https://github.com/lustre-labs/lustre/graphs/contributors&quot;&gt;\n    contributors\n  &lt;/a&gt;\n&lt;/div&gt;\n\n---\n\n## Table of contents\n\n- [Features](#features)\n- [Example](#example)\n- [Philosophy](#philosophy)\n- [Installation](#installation)\n- [Where next](#where-next)\n- [Support](#support)\n\n## Features\n\n- A **declarative**, functional API for constructing HTML. No templates, no macros,\n  just Gleam.\n\n- An Erlang and Elm-inspired architecture for **managing state**.\n\n- **Managed side effects** for predictable, testable code.\n\n- Universal components. **Write once, run anywhere**. Elm meets Phoenix LiveView.\n\n- A **batteries-included CLI** that makes scaffolding and building apps a breeze.\n\n- **Server-side rendering** for static HTML templating.\n\n## Example\n\n```gleam\nimport gleam/int\nimport lustre\nimport lustre/element.{text}\nimport lustre/element/html.{div, button, p}\nimport lustre/event.{on_click}\n\npub fn main() {\n  let app = lustre.simple(init, update, view)\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n\nfn init(_flags) {\n  0\n}\n\ntype Msg {\n  Incr\n  Decr\n}\n\nfn update(model, msg) {\n  case msg {\n    Incr -&gt; model + 1\n    Decr -&gt; model - 1\n  }\n}\n\nfn view(model) {\n  let count = int.to_string(model)\n\n  div([], [\n    button([on_click(Incr)], [text(&quot; + &quot;)]),\n    p([], [text(count)]),\n    button([on_click(Decr)], [text(&quot; - &quot;)])\n  ])\n}\n```\n\n## Philosophy\n\nLustre is an _opinionated_ framework for building small-to-medium-sized Web\napplications. Modern frontend development is hard and complex. Some of that\ncomplexity is necessary, but a lot of it is accidental or comes from having far\ntoo many options. Lustre has the same design philosophy as Gleam: where possible,\nthere should be only one way to do things.\n\nThat means shipping with a single state management system out of the box, modelled\nafter Elm and Erlang/OTP. Open any Lustre application and you should feel\nright at home.\n\nIt also means we encourage simple approaches to constructing views over complex\nones. Lustre _does_ have a way to create encapsulated stateful components (something\nwe sorely missed in Elm) but it shouldn&#39;t be the default. Prefer simple functions\nto stateful components.\n\nWhere components _are_ necessary, lean into the fact that Lustre components can\nrun _anywhere_. Lustre gives you the tools to write components that can run inside\nan existing Lustre application, export them as a standalone Web Component, or run\nthem on the server with a minimal runtime for patching the DOM. Lustre calls these\n**universal components** and they&#39;re written with Gleam&#39;s multiple targets in mind.\n\n## Installation\n\nLustre is published on [Hex](https://hex.pm/packages/lustre)! You can add it to\nyour Gleam projects from the command line:\n\n```sh\ngleam add lustre\n```\n\nLustre also has a companion package containing development tooling that you might\nlike to install:\n\n&gt; **Note**: the lustre_dev_tools development server watches your filesystem for\n&gt; changes to your gleam code and can automatically reload the browser. For linux\n&gt; users this requires [inotify-tools]() be installed\n\n```sh\ngleam add --dev lustre_dev_tools\n```\n\nIf you&#39;re using a different build tool, like Rebar3 or Mix, you can add Lustre\nto your `rebar.config` or `mix.exs` file respectively.\n\n```erlang\n{deps, [\n  {lustre, &quot;4.0.0&quot;}\n]}\n```\n\n```elixir\ndefp deps do\n  [\n    {:lustre, &quot;~&gt; 4.0&quot;}\n  ]\nend\n```\n\n## Where next\n\nTo get up to speed with Lustre, check out the [quickstart guide](https://hexdocs.pm/lustre/guide/01-quickstart.html).\nIf you prefer to see some code, the [examples](https://github.com/lustre-labs/lustre/tree/main/examples)\ndirectory contains a handful of small applications that demonstrate different\naspects of the framework.\n\nYou can also read through the documentation and API reference on\n[HexDocs](https://hexdocs.pm/lustre).\n\n## Support\n\nLustre is mostly built by just me, [Hayleigh](https://github.com/hayleigh-dot-dev),\naround two jobs. If you&#39;d like to support my work, you can [sponsor me on GitHub](https://github.com/sponsors/hayleigh-dot-dev).\n\nContributions are also very welcome! If you&#39;ve spotted a bug, or would like to\nsuggest a feature, please open an issue or a pull request.\n","url":"index.html"},{"doc":"lustre","title":"lustre","content":"# Lustre examples directory\n\nEach of the examples in this directory is a self-contained, complete Gleam app\nthat demonstrates a particular feature or concept of the library. For newcomers,\nwe recommend looking through them in order, as each example tends to build on\nthe previous ones. Feel free to jump in to any example that interests you, though!\n\n&gt; **Note**: these examples all use [`lustre/ui`](https://github.com/lustre-labs/ui)\n&gt; to show off something a little more visually interesting than unstyled HTML. None\n&gt; of the ideas in these examples are specific to `lustre/ui`, though, and you should\n&gt; know that you can follow along with any of these examples using only the standard\n&gt; `lustre/element/html` module.\n\n## Examples\n\n- [`01-hello-world`](https://github.com/lustre-labs/lustre/tree/main/examples/01-hello-world)\n  is a simple example to just get something on the screen.\n\n- [`02-interactivity`](https://github.com/lustre-labs/lustre/tree/main/examples/02-interactivity)\n  introduces the core Model-View-Update loop that underpins every Lustre application.\n\n- [`03-controlled-inputs`](https://github.com/lustre-labs/lustre/tree/main/examples/03-controlled-inputs)\n  demonstrates the most common way to handle `&lt;input /&gt;` elements in Lustre.\n\n- [`04-custom-event-handlers`](https://github.com/lustre-labs/lustre/tree/main/examples/04-custom-event-handlers)\n  shows you how to write your own event handlers and decoders, instead of relying\n  on the ones provided by `lustre/event`.\n\n- [`05-http-requests`](https://github.com/lustre-labs/lustre/tree/main/examples/05-http-requests)\n  demonstrates how side effects are handled in Lustre, using the third-party\n  [`lustre_http`](https://hexdocs.pm/lustre_http/) package.\n\n- [`06-custom-effects`](https://github.com/lustre-labs/lustre/tree/main/examples/06-custom-effects)\n  builds on the previous example and shows you how to write your own side effects\n  for Lustre to perform.\n\n- [`07-routing`](https://github.com/lustre-labs/lustre/tree/main/examples/07-routing)\n  shows how to use [`modem`](https://hexdocs.pm/modem/) to set up routing and\n  navigating between pages in a Lustre app.\n\n## Getting help\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"reference/examples.html"},{"doc":"lustre","title":"lustre","content":"","url":"#"},{"doc":"lustre","title":"lustre","content":"# 01 Quickstart guide\n\nWelcome to the Lustre quickstart guide! This document should get you up to speed\nwith the core ideas that underpin every Lustre application as well as how to get\nsomething on the screen.\n\n## What is a SPA?\n\nLustre can be used to create HTML in many different contexts, but it is primarily\ndesigned to be used to build Single-Page Applications – or SPAs. SPAs are a type\nof Web application that render content primarily in the browser (rather than on\nthe server) and, crucially, do not require a full page load when navigating\nbetween pages or loading new content.\n\nTo help build these kinds of applications, Lustre comes with an opinionated\nruntime. Some of Lustre&#39;s core features include:\n\n- **Declarative rendering**: User interfaces are constructed using a declarative\n  API that describes HTML as a function of your application&#39;s state. This is in\n  contrast to more traditional imperative approaches to direct DOM mutation like\n  jQuery.\n\n- **State management**: If UIs are a function of state, then orchestrating state\n  changes is crucial! Lustre provides a simple message-based state management\n  system modelled after OTP [gen_servers](https://www.erlang.org/doc/design_principles/gen_server_concepts),\n  Gleam&#39;s [actors](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html), and the\n  [Elm Architecture](https://guide.elm-lang.org/architecture/).\n\n- **Managed side effects**: Managing asynchronous operations like HTTP requests\n  and timers can be tricky when JavaScript is single-threaded. Lustre provides a\n  runtime to manage these side effects and let them communicate with your application\n  using the same messages as your update loop.\n\n## Your first Lustre program\n\nTo get started, let&#39;s create a new Gleam application and add Lustre as a dependency.\n\n```sh\ngleam new app &amp;&amp; cd app &amp;&amp; gleam add lustre\n```\n\nBy default, Gleam builds projects for the Erlang target unless told otherwise. We\ncan change this by adding a `target` field to the `gleam.toml` file generated in\nthe root of the project.\n\n```diff\n  name = &quot;app&quot;\n+ target = &quot;javascript&quot;\n  version = &quot;1.0.0&quot;\n\n  ...\n```\n\nThe simplest type of Lustre application is constructed with the `element` function.\nThis produces an application that renders a static piece of content without the\ntypical update loop.\n\nWe can start by importing `lustre` and `lustre/element` and just rendering some\ntext:\n\n```gleam\nimport lustre\nimport lustre/element\n\npub fn main() {\n  let app = lustre.element(element.text(&quot;Hello, world!&quot;))\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n```\n\nLustre has some official development tooling published in the\n[`lustre_dev_tools`](https://hexdocs.pm/lustre_dev_tools/) package. Most projects\nwill probably want to add those too!\n\n&gt; **Note**: the lustre_dev_tools development server watches your filesystem for\n&gt; changes to your gleam code and can automatically reload the browser. For linux\n&gt; users this requires [inotify-tools](https://github.com/inotify-tools/inotify-tools)\n&gt; be installed. If you do not or cannot install this, the development server will\n&gt; still run but it will not watch your files for changes.\n\n```sh\ngleam add --dev lustre_dev_tools\n```\n\nIt&#39;s important to make sure the development tooling is added as a `--dev`\ndependency. This ensures they&#39;re never included in production builds of your app.\n\nTo start a development server, we can run:\n\n```sh\ngleam run -m lustre/dev start\n```\n\nThe first time you run this command might take a little while, but subsequent runs\nshould be much faster!\n\n&gt; **Note**: Lustre uses esbuild under the hood, and attempts to download the [right\n&gt; binary for your platform](https://esbuild.github.io/getting-started/#download-a-build).\n&gt; If you&#39;re not connected to the internet, on an unsupported platform, or don&#39;t\n&gt; want Lustre to download the binary you can grab or build it yourself and place it\n&gt; in `build/.lustre/bin/esbuild`.\n\nOnce the server is up and running you should be able to visit http://localhost:1234\nand be greeted with your &quot;Hello, world!&quot; message.\n\nWe mentioned Lustre has a declarative API for constructing HTML. Let&#39;s see what\nthat looks like by building something slightly more complex.\n\n```gleam\nimport lustre\nimport lustre/attribute\nimport lustre/element\nimport lustre/element/html\n\npub fn main() {\n  let app =\n    lustre.element(\n      html.div([], [\n        html.h1([], [element.text(&quot;Hello, world!&quot;)]),\n        html.figure([], [\n          html.img([attribute.src(&quot;https://cdn2.thecatapi.com/images/b7k.jpg&quot;)]),\n          html.figcaption([], [element.text(&quot;A cat!&quot;)])\n        ])\n      ])\n    )\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n```\n\nHere we _describe_ the structure of the HTML we want to render, and leave the\nbusywork to Lustre&#39;s runtime: that&#39;s what makes it declarative!\n\n&quot;**Where are the templates?**&quot; we hear you cry. Lustre doesn&#39;t have a separate\ntemplating syntax like JSX or HEEx for a few reasons (lack of metaprogramming\nbuilt into Gleam, for one). Some folks might find this a bit odd at first, but\nwe encourage you to give it a try. Realising that your UI is _just functions_\ncan be a bit of a lightbulb moment as you build more complex applications.\n\n## Adding interactivity\n\nRendering static HTML is great, but we said at the beginning Lustre was designed\nprimarily for building SPAs – and SPAs are interactive! To do that we&#39;ll need\nto move on from `lustre.element` to the first of Lustre&#39;s application constructors\nthat includes an update loop: `lustre.simple`.\n\n```gleam\nimport gleam/int\nimport lustre\nimport lustre/element\nimport lustre/element/html\nimport lustre/event\n\npub fn main() {\n  let app = lustre.simple(init, update, view)\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n```\n\nThere are three main building blocks to every interactive Lustre application:\n\n- A `Model` that represents your application&#39;s state and an `init` function\n  to create it.\n\n- A `Msg` type that represents all the different ways the outside world can\n  communicate with your application and an `update` function that modifies\n  your model in response to those messages.\n\n- A `view` function that renders your model to HTML.\n\nWe&#39;ll build a simple counter application to demonstrate these concepts. Our\nmodel can be an `Int` and our `init` function will initialise it to `0`:\n\n```gleam\npub type Model = Int\n\nfn init(_flags) -&gt; Model {\n  0\n}\n```\n\n&gt; **Note**: The `init` function always takes a single argument! These are the &quot;flags&quot;\n&gt; or start arguments you can pass in when your application is started with\n&gt; `lustre.start`. For the time being, we can ignore them, but they&#39;re useful for\n&gt; passing in configuration or other data when your application starts.\n\nThe main update loop in a Lustre application revolves around messages passed in\nfrom the outside world. For our counter application, we&#39;ll have two messages to\nincrement and decrement the counter:\n\n```gleam\npub type Msg {\n  Increment\n  Decrement\n}\n\npub fn update(model: Model, msg: Msg) -&gt; Model {\n  case msg {\n    Increment -&gt; model + 1\n    Decrement -&gt; model - 1\n  }\n}\n```\n\nEach time a message is produced from an event listener, Lustre will call your\n`update` function with the current model and the incoming message. The result\nwill be the new application state that is then passed to the `view` function:\n\n```gleam\npub fn view(model: Model) -&gt; element.Element(Msg) {\n  let count = int.to_string(model)\n\n  html.div([], [\n    html.button([event.on_click(Increment)], [\n      element.text(&quot;+&quot;)\n    ]),\n    element.text(count),\n    html.button([event.on_click(Decrement)], [\n      element.text(&quot;-&quot;)\n    ])\n  ])\n}\n```\n\nThe above snippet attaches two click event listeners that produce an `Increment`\nor `Decrement` message when clicked. The Lustre runtime is responsible for\nattaching these event listeners and calling your `update` function with the\nresulting message.\n\n&gt; **Note**: notice that the return type of `view` is `element.Element(Msg)`. The\n&gt; type parameter `Msg` tells us the kinds of messages this element might produce\n&gt; from events: type safety to the rescue!\n\nThis forms the core of every Lustre application:\n\n- A model produces some view.\n- The view can produce messages in response to user interaction.\n- Those messages are passed to the update function to produce a new model.\n- ... and the cycle continues.\n\n## Talking to the outside world\n\nThis &quot;closed loop&quot; of messages and updates works well if all we need is an\ninteractive document, but many applications will also need to talk to the outside\nworld – whether that&#39;s fetching data from an API, setting up a WebSocket connection,\nor even just setting a timer.\n\nLustre manages these side effects through an abstraction called an `Effect`. In\nessence, effects are any functions that talk with the outside world and might\nwant to send messages back to your application. Lustre lets you write your own\neffects, but for now we&#39;ll use a community package called\n[`lustre_http`](https://hexdocs.pm/lustre_http/index.html) to fetch a new cat image\nevery time the counter is incremented.\n\nBecause this is a separate package, make sure to add it to your project first.\n\n```sh\n$ gleam add lustre_http\n```\n\nNow we are introducing side effects, we need to graduate from `lustre.simple` to\nthe more powerful `lustre.application` constructor.\n\n```gleam\nimport gleam/dynamic\nimport gleam/int\nimport gleam/list\nimport lustre\nimport lustre/attribute\nimport lustre/effect\nimport lustre/element\nimport lustre/element/html\nimport lustre/event\nimport lustre_http\n\npub fn main() {\n  let app = lustre.application(init, update, view)\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n```\n\nIf you edited your previous counter app, you&#39;ll notice the program no longer\ncompiles. Specifically, the type of our `init` and `update` functions are wrong\nfor the new `lustre.application` constructor!\n\nIn order to tell Lustre about what effects it should perform, these functions now\nneed to return a _tuple_ of the new model and any effects. We can amend our `init`\nfunction like so:\n\n```gleam\npub type Cat {\n  Cat(id: String, url: String)\n}\n\npub type Model {\n  Model(count: Int, cats: List(Cat))\n}\n\nfn init(_flags) -&gt; #(Model, effect.Effect(Msg)) {\n  #(Model(0, []), effect.none())\n}\n```\n\nThe `effect.none` function is a way of saying &quot;no effects&quot; – we don&#39;t need to do\nanything when the application starts. We&#39;ve also changed our `Model` type from a\nsimple type alias to a Gleam [record](https://tour.gleam.run/data-types/records/)\nthat holds both the current count and a list of cat image URLs.\n\nIn our `update` function, we want to fetch a new cat image every time the counter\nis incremented. To do this we need two things:\n\n- An `Effect` to describe the request the runtime should perform.\n- A variant of our `Msg` to handle the response.\n\nThe `lustre_http` package has the effect side of things handled, so we just need\nto modify our `Msg` type to include a new variant for the response:\n\n```gleam\npub type Msg {\n  UserIncrementedCount\n  UserDecrementedCount\n  ApiReturnedCats(Result(List(Cat), lustre_http.HttpError))\n}\n```\n\n&gt; **Note**: Concerned your message type is too verbose? Read our thoughts on why\n&gt; this is a good thing in our [state management guide](./02-state-management.html).\n\nFinally, we can modify our `update` function to also fetch a cat image when the\ncounter is incremented and handle the response:\n\n```gleam\npub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n  case msg {\n    UserIncrementedCount -&gt; #(Model(..model, count: model.count + 1), get_cat())\n    UserDecrementedCount -&gt; #(\n      Model(count: model.count - 1, cats: list.drop(model.cats, 1)),\n      effect.none(),\n    )\n    ApiReturnedCats(Ok(api_cats)) -&gt; {\n      let assert [cat, ..] = api_cats\n      #(Model(..model, cats: [cat, ..model.cats]), effect.none())\n    }\n    ApiReturnedCats(Error(_)) -&gt; #(model, effect.none())\n  }\n}\n\nfn get_cat() -&gt; effect.Effect(Msg) {\n  let decoder =\n    dynamic.decode2(\n      Cat,\n      dynamic.field(&quot;id&quot;, dynamic.string),\n      dynamic.field(&quot;url&quot;, dynamic.string),\n    )\n  let expect = lustre_http.expect_json(dynamic.list(decoder), ApiReturnedCats)\n\n  lustre_http.get(&quot;https://api.thecatapi.com/v1/images/search&quot;, expect)\n}\n```\n\n&gt; **Note**: The `get_cat` function returns an `Effect` that tells the runtime how\n&gt; to fetch a cat image. It&#39;s important to know that the `get_cat` function doesn&#39;t\n&gt; perform the request directly! This is why we need to add the `ApiReturnedCat` message\n&gt; variant: the runtime needs to know what to do with the response when it arrives.\n\nThis model of managed effects can feel cumbersome at first, but it comes with some\nbenefits. Forcing side effects to produce a message means our message type naturally\ndescribes all the ways the world can communicate with our application; as an app\ngrows being able to get this kind of overview is invaluable! It also means we can\ntest our update loop in isolation from the runtime and side effects: we could write\ntests that verify a particular sequence of messages produces an expected model\nwithout needing to mock out HTTP requests or timers.\n\nBefore we forget, let&#39;s also update our `view` function to actually display the\ncat images we&#39;re fetching:\n\n```gleam\npub fn view(model: Model) -&gt; element.Element(Msg) {\n  let count = int.to_string(model.count)\n\n  html.div([], [\n    html.button([event.on_click(UserIncrementedCount)], [\n      element.text(&quot;+&quot;)\n    ]),\n    element.text(count),\n    html.button([event.on_click(UserDecrementedCount)], [\n      element.text(&quot;-&quot;)\n    ]),\n    html.div(\n      [],\n      list.map(model.cats, fn(cat) {\n        html.img([\n          attribute.src(cat.url),\n          attribute.width(400),\n          attribute.height(400),\n        ])\n      }),\n    ),\n  ])\n}\n```\n\n&gt; **Note**: Depending on how fast the cat images download, and your browser window\n&gt; size and zoom level, you might notice that when you click the increment counter\n&gt; that the last cat image is duplicated before the new image loads. This is expected.\n&gt; To learn more about why this happens and how to prevent this behaviour, see\n&gt; [rendering lists](https://github.com/lustre-labs/lustre/blob/main/pages/hints/rendering-lists.md)\n\n## Where to go from here\n\nBelieve it or not, you&#39;ve already seen about 80% of what Lustre has to offer! From\nthese core concepts, you can build rich interactive applications that are predictable\nand maintainable. Where to go from here depends on what you want to build, and\nhow you like to learn:\n\n- There are a number of [examples](https://github.com/lustre-labs/lustre/tree/main/examples)\n  if the Lustre repository that gradually introduce more complex applications\n  and ideas.\n\n- The [rest of this guide](./02-state-management.html) also continues to teach\n  Lustre&#39;s high-level concepts and best-practices.\n\n- Of course, if you want to dive in and start making things straight away, the\n  [API documentation](https://hexdocs.pm/lustre/lustre.html) is always handy to keep open.\n\n## Getting help\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n\nWhile our docs are still a work in progress, the official [Elm guide](https://guide.elm-lang.org)\nis also a great resource for learning about the Model-View-Update architecture\nand the kinds of patterns that Lustre is built around.\n","url":"guide/01-quickstart.html"},{"doc":"lustre","title":"lustre","content":"# 02 State management\n\nWe saw in the quickstart guide that all Lustre applications are built around the\nModel-View-Update (MVU) architecture. This means that the state of the application\nis stored in a single, immutable data structure called the model, and updated as\nmessages are dispatched to the runtime.\n\nThe MVU architecture is an example of _unidirectional data flow_:\n\n- Your model describes the entire state of your application at a given point in\n  time.\n\n- The UI is a [pure](https://github.com/lustre-labs/lustre/blob/main/pages/hints/pure-functions.md)\n  function of that model: if the model doesn&#39;t change, the UI doesn&#39;t change.\n\n- Events from the outside world – user interaction, HTTP responses, ... – send\n  messages to an update function that constructs a new model.\n\n- The UI re-renders based on the new state.\n\n```text\n                                       +--------+\n                                       |        |\n                                       | update |\n                                       |        |\n                                       +--------+\n                                         ^    |\n                                         |    |\n                                     Msg |    | Model\n                                         |    |\n                                         |    v\n+------+                         +------------------------+\n|      |          Model          |                        |\n| init |------------------------&gt;|     Lustre Runtime     |\n|      |                         |                        |\n+------+                         +------------------------+\n                                         ^    |\n                                         |    |\n                                     Msg |    | Model\n                                         |    |\n                                         |    v\n                                       +--------+\n                                       |        |\n                                       |  view  |\n                                       |        |\n                                       +--------+\n```\n\nThis is in contrast to _bidirectional_ approaches to state management, where the\nUI can modify state directly. For some developers this can be a difficult idea\nto get used to, but it brings a number of benefits:\n\n- A **single source of truth** makes it easier to reason about the state of your\n  application. State management is lifted _out_ of UI code, letting it focus just\n  on presentation and making it easier to test and refactor.\n\n- Message-driven **declarative state updates** give you a holistic view of how\n  your application can change over time. Tracking incoming messages gives you a\n  history of state updates and can be serialised and logged for debugging or\n  testing purposes.\n\n- State updates are **pure**. We will learn more about this in the [next guide](./03-side-effects.html),\n  but for now it is enough to know that this means testing your state changes is\n  much easier because mocking messages is simpler than mocking side effects!\n\nThe rest of this guide contains some learned-wisdom and best practices for managing\nstate in Lustre applications.\n\n## The best model is not always a record\n\nIt is overwhelmingly common to see the model of a Lustre application as a single\nrecord. This is a sensible place to start, but there are other options! Gleam&#39;s\ncustom types allow us to model our data as disjoint variants. Using these as your\napplication&#39;s model can be particularly useful when you have different states that\ndo not need to persist across navigations:\n\n```gleam\ntype Model {\n  LoggedIn(LoggedInModel)\n  Public(PublicModel)\n}\n\ntype LoggedInModel {\n  ...\n}\n\ntype PublicModel {\n  ...\n}\n```\n\nHere, we have a model that represents our application as either having a logged in\nuser or just one of the public routes. This pushes us towards the great practice of\n[making impossible states impossible](https://github.com/stereobooster/pragmatic-types/blob/master/posts/making-impossible-states-impossible.md).\nNow, we can write separate update and view functions that only handle the states\nthey care about.\n\nAnother option is to use a _type alias_ to represent some state using existing\nGleam types. It&#39;s important to remember that your model represents _application_\nstate and not necessarily _page_ state. This can manifest as simple as aliasing\nGleam&#39;s `Result` type or maybe a `Dict` representing loaded posts.\n\n## Messages not actions\n\nLustre is not the first frontend framework to use the MVU architecture or to\nfocus on dispatching messages to update state. State management libraries like\nRedux and Zustand follow a very similar pattern. The devil is in the details\nthough, and these libraries often talk in terms of _actions_ but you&#39;ll see\nElm and Lustre prefer the term _message_.\n\nActions frame incoming events as _things to do_: &quot;add a new todo&quot;, &quot;make an HTTP\nrequest&quot;, etc. This can work well in the beginning, but as your application grows\nand the number of things you can do grows, naming messages as actions can become\nproblematic.\n\nIn particular, it encourages you to recursively call your `update` function with\ndifferent messages when you want to compose behaviour. Gleam is a functional\nprogramming language: we should use functions to update our state, not message\ndispatching! Communicating through messages is a way for the _outside world_ to\ntalk to our application, not for our applications to talk to themselves.\n\nA recursive update function makes it difficult to see the consequences of any one\nmessage as you need to trace through the recursive calls in your head to understand\nwhich messages are being dispatched and in what order.\n\nInstead, we recommend you name your messages according to a **Subject Verb Object**\npattern. This frames messages based on who (or what) sent them, what state or\n&quot;thing&quot; they&#39;re working on, and what they did or want to do. Imagine a password\nreset form, the user can type in a new password and submit it and our app waits\nfor a response. As a first-pass we might end up with something like this:\n\n```gleam\ntype Msg {\n  SetPassword(String)\n  ResetPassword\n  PasswordReset(Result(Nil, String))\n}\n```\n\nThis is quite muddled, and is compounded as we add more messages to our app\n(especially if they also relate to the password!). It&#39;s hard to tell from looking\nat our messages what our app might _really_ be doing: we&#39;d have to dig into our\n`update` function and possibly our `view` to work out what our intent was. One\nsuper power of the MVU pattern is that we can look at our messages to get a\nholistic view of what our app can handle. Things become much clearer if we refactor\nthis example to the Subject Verb Object naming pattern:\n\n```gleam\ntype Msg {\n  UserUpdatedPassword(String)\n  UserRequestedPasswordReset\n  BackendResetPassword(Result(Nil, String))\n}\n```\n\nIt&#39;s now immediately obvious at a glance:\n\n1. Where these messages are coming from (user interaction, the network, ...)\n2. What sort of event or intention they represent\n\nAs our apps grow in size, we&#39;ll be thankful for this clarity!\n\n## View functions not components\n\nAlthough Lustre does have a way to create encapsulated stateful components (something\nwe sorely missed in Elm) it shouldn&#39;t be the default. The word &quot;component&quot; is a bit\noverloaded in the frontend world, so for clarity Lustre considers _components_\nas stateful nested Model-View-Update applications and calls stateless functions\nthat return `Element`s _view functions_.\n\nThe best Lustre code bases take the lessons learned from similar languages like\nElm, Erlang, and Elixir and keep the number of components low and the number of\nsimple view functions much higher. If you&#39;re coming from a typical frontend\nframework the idea of eschewing stateful components might seem quite strange, but\nthere are some tangible benefits to this approach:\n\n- **Favouring view functions forces us to be intentional with state.**\n\n  Frameworks often make it easy to add state to components, which in turn makes\n  it easy to add state without really thinking about whether we need it or whether\n  we&#39;re taking the best approach.\n\n  View functions on the other hand _only_ have arguments, and adding a new argument\n  is a much more deliberate act. This gives us a chance to consider whether we&#39;re\n  modelling things the right way or whether we&#39;re trying to do too much.\n\n- **Components are bad for code organisation.**\n\n  It can be tempting to use components as a way to organise code. You might see\n  this commonly in React and Vue codebases: you have a folder for components, a\n  folder for hooks, and so on. Using components as a means of organisation often\n  leads to us drawing weird boundaries around our code and spreading out things\n  that should be together.\n\n  By sticking to view functions we&#39;re much more likely to keep code grouped based\n  on _what it does_ rather than what it _is_ and this approach is much more idiomatic\n  to Gleam on the whole, and also an approach favoured by Elm and Elixir alike.\n\n- **Avoiding components makes your code easier to test.**\n\n  When we reach for components too soon or too frequently, we often end up needing\n  to pull in a complete E2E testing framework to make sure our code is behaving\n  correctly, or we might end up exposing our components&#39; internals for testing:\n  defeating the purpose of encapsulation in the first place!\n\n  By sticking to plain view functions and functions to transform data before\n  rendering, we end up with a codebase that is much easier to test with Gleam&#39;s\n  available testing tools.\n\n- **Overusing components makes refactoring more challenging.**\n\n  Imagine you have a table component with tabs to switch between different views.\n  If some time in the future you decide to pull the tabs out so they can be\n  rendered elsewhere on the page you&#39;ll discover that the tabs&#39; state was tightly\n  coupled to the table. Now we are forced to refactor the table component so the\n  tab state can be passed in as an attribute. We&#39;ll also need to refactor the\n  _parent_ to contain the state of the tabs so it can be passed down to both\n  components.\n\n  By avoiding components this sort of refactoring becomes simpler: we were already\n  managing the state further up the component tree so moving things around is\n  much less painful.\n\n- **Creating components is more boilerplate.**\n\n  Components share the same shape as any other Lustre application. That means for\n  any component you want to create, you also need to define an `init`, `update`,\n  and `view` function, a `Model` type, and a `Msg` type. If you find yourself\n  thinking &quot;wow, this is a lot of boilerplate just to do X&quot; then listen to your\n  gut!\n\n## Related examples\n\nIf you&#39;d like to see some of the ideas in action, we have a number of examples\nthat demonstrate how to use Lustre in practice:\n\n- [`02-interactivity`](https://github.com/lustre-labs/lustre/tree/main/examples/02-interactivity)\n- [`03-controlled-inputs`](https://github.com/lustre-labs/lustre/tree/main/examples/03-controlled-inputs)\n\n## Getting help\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"guide/02-state-management.html"},{"doc":"lustre","title":"lustre","content":"# 03 Side effects\n\nLustre&#39;s implementation of the Model-View-Update architecture includes one\nadditional piece of the puzzle: managed side effects. If we take the MVU diagram\nfrom the previous guide and upgrade it to include managed effects, it looks like\nthis:\n\n```text\n                                       +--------+\n                                       |        |\n                                       | update |\n                                       |        |\n                                       +--------+\n                                         ^    |\n                                         |    |\n                                     Msg |    | #(Model, Effect(msg))\n                                         |    |\n                                         |    v\n+------+                         +------------------------+\n|      |  #(Model, Effect(msg))  |                        |\n| init |------------------------&gt;|     Lustre Runtime     |\n|      |                         |                        |\n+------+                         +------------------------+\n                                         ^    |\n                                         |    |\n                                     Msg |    | Model\n                                         |    |\n                                         |    v\n                                       +--------+\n                                       |        |\n                                       |  view  |\n                                       |        |\n                                       +--------+\n```\n\nWell what does managed effects mean, exactly? In Lustre, we expect your `init`,\n`update`, and `view` functions to be [_pure_](https://github.com/lustre-labs/lustre/blob/main/pages/hints/pure-functions.md).\nThat means they shouldn&#39;t perform side effects like making an HTTP request or writing\nto local storage: we should be able to run your functions 100 times with the same\ninput and get the same output every time!\n\nOf course, in real applications performing HTTP requests and writing to local\nstorage turn out to be quite useful things to do. If we shouldn&#39;t perform side\neffects in our code how do we do them then? Lustre has an [`Effect`](https://hexdocs.pm/lustre/lustre/effect.html)\ntype that _tells the runtime what side effects to perform_. So we say &quot;Hey, I\nwant to make an HTTP request to this URL and when you get the response, dispatch\nthis message to me&quot;. The runtime takes care of performing the side effect and\nturning the result into something our `update` function understands.\n\n## Why managed effects?\n\nThis can feel like a lot of ceremony to go through just to make an HTTP request.\nThe natural question is: why not just let us make these requests ourselves?\n\nManaged effects have a number of benefits that come from _separating our programs\nfrom the outside world_:\n\n1. **Predictability**: by keeping side effects out of our `update` function, we\n   can be confident that our application&#39;s state is only ever changed in one\n   place. This makes it easier to reason about our code and track down bugs.\n\n2. **Testability**: because our application code is pure, we can test it without\n   needing to mock out HTTP services or browser APIs. We can test our `update`\n   function, for example, by passing in a sequence of messages: no network mocks\n   required!\n\n3. **Reusability**: Lustre applications can run in a variety of environments and\n   contexts. The more we push platform-specific code into managed effects, the\n   easier time we&#39;ll have running our application as a [server component](https://hexdocs.pm/lustre/lustre/server_component.html)\n   or as a static site.\n\n## Packages for common effects\n\nThe community has started to build packages that cover common side effects. For\nmany applications it&#39;s enough to drop these packages in and start using them\nwithout needing to write any custom effects.\n\n&gt; **Note**: _all_ of these packages are community maintained and unrelated to the\n&gt; core Lustre organisation. If you run into issues please open an issue on the\n&gt; package&#39;s repository!\n\n- [`lustre_http`](https://hexdocs.pm/lustre_http/) lets you make HTTP requests\n  and describe what responses to expect from them.\n\n- [`lustre_websocket`](https://hexdocs.pm/lustre_websocket/) handles WebSocket\n  connections and messages.\n\n- [`modem`](https://hexdocs.pm/modem/) and [`lustre_routed`](https://hexdocs.pm/lustre_routed/)\n  are two packages that help you manage navigation and routing.\n\n- [`lustre_animation`](https://hexdocs.pm/lustre_animation/) is a simple package\n  for interpolating between values over time.\n\n## Running effects\n\nWe know that effects need to be performed by the runtime, but how does the runtime\nknow when we want it to run an effect? If you have been using the `lustre.simple`\napplication constructor until now, it is time to upgrade to\n[`lustre.application`](https://hexdocs.pm/lustre/lustre.html#application)!\n\nFull Lustre applications differ from simple applications in one important way by\nreturning a tuple of `#(Model, Effect(Msg))` from your `init` and `update`\nfunctions:\n\n```gleam\npub fn simple(\n  init: fn(flags) -&gt; model,\n  update: fn(model, msg) -&gt; model,\n  view: fn(model) -&gt; Element(msg),\n) -&gt; App(flags, model, msg)\n\npub fn application(\n  init: fn(flags) -&gt; #(model, Effect(msg)),\n  update: fn(model, msg) -&gt; #(model, Effect(msg)),\n  view: fn(model) -&gt; Element(msg),\n) -&gt; App(flags, model, msg)\n```\n\nWe can, for example, launch an HTTP request on application start by using `lustre_http.get`\nin our `init` function:\n\n```gleam\nfn init(_flags) {\n  let model = Model(...)\n  let get_ip = lustre_http.get(\n    &quot;https://api.ipify.org&quot;,\n    ApiReturnedIpAddress\n  )\n\n  #(model, get_ip)\n}\n```\n\n&gt; **Note**: to tell the runtime we _don&#39;t_ want to perform any side effects this\n&gt; time, we can use [`effect.none()`](https://hexdocs.pm/lustre/lustre/effect.html#none).\n\n## Writing your own effects\n\nWhen you need to do something one of the existing packages doesn&#39;t cover, you need\nto write your own effect. You can do that by passing a callback to\n[`effect.from`](https://hexdocs.pm/lustre/lustre/effect.html#from). Custom effects\nare called with an argument – commonly called `dispatch` – that you can use to\nsend messages back to your application&#39;s `update` function.\n\nBelow is an example of a custom effect that reads a value from local storage:\n\n```js\n// ffi.mjs\nimport { Ok, Error } from &quot;./gleam.mjs&quot;;\n\nexport function read(key) {\n  const value = window.localStorage.getItem(key);\n  return value ? new Ok(value) : new Error(undefined);\n}\n```\n\n```gleam\nfn read(key: String, to_msg: fn(Result(String, Nil)) -&gt; msg) -&gt; Effect(msg) {\n  effect.from(fn(dispatch) {\n    do_read(key)\n    |&gt; to_msg\n    |&gt; dispatch\n  })\n}\n\n@external(javascript, &quot;ffi.mjs&quot;, &quot;read&quot;)\nfn do_read(key: String) -&gt; Result(String, Nil) {\n  Error(Nil)\n}\n```\n\n&gt; **Note**: we provide a default implementation of the `do_read` function that\n&gt; always fails. Where possible it&#39;s good to provide an implementation for all of\n&gt; Gleam&#39;s targets. This makes it much easier to run your code as a\n&gt; [server component](https://hexdocs.pm/lustre/lustre/server_component.html) in\n&gt; the future.\n\n### Effects that touch the DOM\n\nLustre runs all your side effects after your `update` function returns but _before_\nyour `view` function is called. A common bug folks run into is trying to interact\nwith a particular element in the DOM before it&#39;s had a chance to render. As a\nrule of thumb, you should _always_ wrap custom effects that interact with the DOM\nin a [`requestAnimationFrame`](https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame)\ncall to ensure the DOM has had a chance to update first.\n\n### Effects without dispatch\n\nSo far, we have seen side effects that are expected to _return something_ to our\nprogram. If we fire an HTTP request, it wouldn&#39;t be much use if we couldn&#39;t get\nthe response back! Sometimes folks wrongly assume effects _must_ use the `dispatch`\nfunction they&#39;re given, but this isn&#39;t true!\n\nIt&#39;s also totally valid to write effects that don&#39;t dispatch any messages. Earlier\nwe saw an example of how to read from local storage, we might also want an effect\nto _write_ to local storage and there&#39;s not much to dispatch in that case!\n\n```js\n// ffi.mjs\nexport function write(key, value) {\n  window.localStorage.setItem(key, value);\n}\n```\n\n```gleam\n// app.gleam\nfn write(key: String, value: String) -&gt; Effect(msg) {\n  effect.from(fn(_) {\n    do_write(key, value)\n  })\n}\n\n@external(javascript, &quot;ffi.mjs&quot;, &quot;write&quot;)\nfn do_write(key: String, value: String) -&gt; Nil {\n  Nil\n}\n```\n\n### Effects with multiple dispatch\n\nSimilar to effects that don&#39;t dispatch any messages, some folks skip over the fact\neffects can dispatch _multiple_ messages. Packages like [`lustre_websocket`](https://hexdocs.pm/lustre_websocket/)\nand [`modem`](https://hexdocs.pm/modem/) set up effects that will dispatch many\nmessages over the lifetime of your program.\n\nOnce you have a reference to that `dispatch` function, you&#39;re free to call it as\nmany times as you want!\n\n```js\n// ffi.mjs\nexport function every(interval, cb) {\n  window.setInterval(cb, interval);\n}\n```\n\n```gleam\n// app.gleam\nfn every(interval: Int, tick: msg) -&gt; Effect(msg) {\n  effect.from(fn(dispatch) {\n    do_every(interval, fn() {\n      dispatch(tick)\n    })\n  })\n}\n\n@external(javascript, &quot;ffi.mjs&quot;, &quot;every&quot;)\nfn do_every(interval: Int, cb: fn() -&gt; Nil) -&gt; Nil {\n  Nil\n}\n```\n\nHere we set up an effect that will continuously dispatch a `tick` message at a\nfixed interval.\n\n## Related examples\n\nIf you&#39;d like to see some of the ideas in action, we have a number of examples\nthat demonstrate how Lustre&#39;s effects system works in practice:\n\n- [`05-http-requests`](https://github.com/lustre-labs/lustre/tree/main/examples/05-http-requests)\n- [`06-custom-effects`](https://github.com/lustre-labs/lustre/tree/main/examples/06-custom-effects)\n- [`07-routing`](https://github.com/lustre-labs/lustre/tree/main/examples/07-routing)\n\n## Getting help\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"guide/03-side-effects.html"},{"doc":"lustre","title":"lustre","content":"# 05 Server-side rendering\n\nUp until now, we have focused on Lustre&#39;s ability as a framework for building\nSingle Page Applications (SPAs). While Lustre&#39;s development and feature set is\nprimarily focused on SPA development, that doesn&#39;t mean it can&#39;t be used on the\nbackend as well! In this guide we&#39;ll set up a small [mist](https://hexdocs.pm/mist/)\nserver that renders some static HTML using Lustre.\n\n## Setting up the project\n\nWe&#39;ll start by adding the dependencies we need and scaffolding the HTTP server.\nBesides Lustre and Mist, we also need `gleam_erlang` (to keep our application\nalive) and `gleam_http` (for types and functions to work with HTTP requests and\nresponses):\n\n```sh\ngleam new app &amp;&amp; cd app &amp;&amp; gleam add gleam_erlang gleam_http lustre mist\n```\n\nBesides imports for `mist` and `gleam_http` modules, we also need to import some\nmodules to render HTML with Lustre. Importantly, we _don&#39;t_ need anything from the\nmain `lustre` module: we&#39;re not building an application with a runtime!\n\n```gleam\nimport gleam/bytes_builder\nimport gleam/erlang/process\nimport gleam/http/request.{type Request}\nimport gleam/http/response.{type Response}\nimport lustre/element\nimport lustre/element/html.{html}\nimport mist.{type Connection, type ResponseData}\n```\n\nWe&#39;ll modify Mist&#39;s example and write a simple request handler that responds to\nrequests to `/greet/:name` with a greeting message:\n\n```gleam\npub fn main() {\n  let empty_body = mist.Bytes(bytes_builder.new())\n  let not_found = response.set_body(response.new(404), empty_body)\n\n  let assert Ok(_) =\n    fn(req: Request(Connection)) -&gt; Response(ResponseData) {\n      case request.path_segments(req) {\n        [&quot;greet&quot;, name] -&gt; greet(name)\n        _ -&gt; not_found\n      }\n    }\n    |&gt; mist.new\n    |&gt; mist.port(3000)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n```\n\nLet&#39;s take a peek inside that `greet` function:\n\n```gleam\nfn greet(name: String) -&gt; Response(ResponseData) {\n  let res = response.new(200)\n  let html =\n    html([], [\n      html.head([], [html.title([], &quot;Greetings!&quot;)]),\n      html.body([], [\n        html.h1([], [html.text(&quot;Hey there, &quot; &lt;&gt; name &lt;&gt; &quot;!&quot;)])\n      ])\n    ])\n\n  response.set_body(res,\n    html\n    |&gt; element.to_document_string\n    |&gt; bytes_builder.from_string\n    |&gt; mist.Bytes\n  )\n}\n```\n\nThe `lustre/element` module has functions for rendering Lustre elements to a\nstring (or string builder); the `to_document_string` function helpfully prepends\nthe `&lt;!DOCTYPE html&gt;` declaration to the output.\n\nIt&#39;s important to realise that `element.to_string` and `element.to_document_string`\ncan render _any_ Lustre element! This means you could take the `view` function\nfrom your client-side SPA and render it server-side, too.\n\n## Hydration\n\nIf we know we can render our apps server-side, the next logical question is how\ndo we handle _hydration_? Hydration is the process of taking the static HTML\ngenerated by the server and turning it into a fully interactive client application,\nideally doing as little work as possible.\n\nMost frameworks today support hydration or some equivalent, for example by\nserialising the state of each component into the HTML and then picking up where\nthe server left off. Lustre doesn&#39;t have a built-in hydration mechanism, but\nbecause of the way it works, it&#39;s easy to implement one yourself!\n\nWe&#39;ve said many times now that in Lustre, your `view` is just a\n[pure function](https://github.com/lustre-labs/lustre/blob/main/pages/hints/pure-functions.md)\nof your model. We should produce the same HTML every time we call `view` with the\nsame model, no matter how many times we call it.\n\nLet&#39;s use that to our advantage! We know our app&#39;s `init` function is responsible\nfor producing the initial model, so all we need is a way to make sure the initial\nmodel on the client is the same as what the server used to render the page.\n\n```gleam\npub fn view(model: Int) -&gt; Element(Msg) {\n  let count = int.to_string(model)\n\n  html.div([], [\n    html.button([event.on_click(Decr)], [html.text(&quot;-&quot;)]),\n    html.button([event.on_click(Incr)], [html.text(&quot;+&quot;)]),\n    html.p([], [html.text(&quot;Count: &quot; &lt;&gt; count)])\n  ])\n}\n```\n\nWe&#39;ve seen the counter example a thousand times over now, but it&#39;s a good example\nto show off how simple hydration can be. The `view` function produces some HTML\nwith events attached, but we already know Lustre can render _any_ element to a\nstring so that shouldn&#39;t be a problem.\n\nLet&#39;s imagine our HTTP server responds with the following HTML:\n\n```gleam\nimport app/counter\nimport gleam/bytes_builder\nimport gleam/http/response.{type Response}\nimport gleam/json\nimport lustre/attribute\nimport lustre/element.{type Element}\nimport lustre/element/html.{html}\nimport mist.{type ResponseData}\n\nfn app() -&gt; Response(ResponseData) {\n  let res = response.new(200)\n\n  let model = 5\n  let html =\n    html([], [\n      html.head([], [\n        html.script([attribute.type_(&quot;module&quot;), attribute.src(&quot;...&quot;)], &quot;&quot;),\n        html.script([attribute.type_(&quot;application/json&quot;), attribute.id(&quot;model&quot;)],\n          json.int(model)\n          |&gt; json.to_string\n        )\n      ]),\n      html.body([], [\n        html.div([attribute.id(&quot;app&quot;)], [\n          counter.view(model)\n        ])\n      ])\n    ])\n\n  response.set_body(res,\n    html\n    |&gt; element.to_document_string\n    |&gt; bytes_builder.from_string\n    |&gt; mist.Bytes\n  )\n}\n```\n\nWe&#39;ve rendered the shell of our application, as well as the counter using `5` as\nthe initial model. Importantly, we&#39;ve included a `&lt;script&gt;` tag with the initial\nmodel encoded as JSON (it might just be an `Int` in this example, but it could\nbe anything).\n\nOn the client, it&#39;s a matter of reading that JSON and decoding it as our initial\nmodel. The [plinth](https://hexdocs.pm/plinth/plinth.html) package provides\nbindings to many browser APIs, we can use that to read the JSON out of the script\ntag:\n\n```gleam\nimport gleam/dynamic\nimport gleam/json\nimport gleam/result\nimport lustre\nimport plinth/browser/document\nimport plinth/browser/element\n\npub fn main() {\n  let json =\n    document.query_selector(&quot;#model&quot;)\n    |&gt; result.map(element.inner_text)\n\n  let flags =\n    case json.decode_string(json, dynamic.int) {\n      Ok(count) -&gt; count\n      Error(_) -&gt; 0\n    }\n\n  let app = lustre.application(init, update, view)\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, flags)\n}\n```\n\nHey that wasn&#39;t so bad! We made sure to fall back to an initial count of `0` if\nwe failed to decode the JSON: this lets us handle cases where the server might\nnot want us to hydrate.\n\nIf you were to set this all up, run it, and check your browser&#39;s developer tools,\nyou&#39;d see that the existing HTML was not replaced and the app is fully interactive!\n\nFor many cases serialising the entire model will work just fine. But remember\nthat Lustre&#39;s super power is that pure `view` function. If you&#39;re smart, you can\nreduce the amount of data you serialise and _derive_ the rest of your model from\nthat.\n\nWe brushed over quite a few details showing how hydration could work here, but in\nthe [next guide](https://hexdocs.pm/lustre/guide/06-full-stack-applications.html)\nwe&#39;ll go into a lot more detail on how to set up and run a full-stack Lustre app.\n\n## Getting help\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"guide/05-server-side-rendering.html"},{"doc":"lustre","title":"lustre","content":"&gt; **Note**: this guide is a work in progress and is not currently complete. Content\n&gt; here will change and be added over time. In the meantime, you can check out the\n&gt; [Gleam Discord server](https://discord.gg/Fm8Pwmy) if you have any questions about\n&gt; full stack applications with Lustre.\n\n# 06 Full stack applications\n\nWe&#39;ve now seen how Lustre can render single-page applications in the browser,\nstatic HTML templates, and we&#39;ve seen how hydration can be implemented. In this\nguide we&#39;ll look at how to put these pieces together into a single application.\n\nTo create a full stack Web application in Gleam you will need to adopt a\n_monorepo_. Although Gleam supports multiple targets, and has conditional\ncompilation features, the language isn&#39;t designed to support a single codebase\nwith different applications for different targets. Instead, we will create\n_three separate Gleam projects_:\n\n```sh\nmkdir lustre-fullstack-guide \\\n  &amp;&amp; gleam new client --name app \\\n  &amp;&amp; gleam new server --name app\n```\n\nWe have one project for the frontend SPA, one project for the server, and a third\nproject called `shared` that we will use to share types and code across the stack.\n\nFull stack applications can get quite complex quite quickly, so this guide will\nfocus on putting together a simple grocery list with pre-rendering and hydration,\nas well as a single API endpoint to save the items to a SQLite database.\n\n## The client\n\nThe application we&#39;re building will eventually need to make requests to our backend\nAPI, which means we need an application capable of performing effects. To start,\nlet&#39;s scaffold out all the types and functions we need:\n\n```sh\ngleam add decipher lustre lustre_http gleam_json\n```\n\n```sh\ngleam add lustre_dev_tools --dev\n```\n\n```gleam\nimport gleam/dynamic\nimport gleam/int\nimport gleam/result\nimport lustre\nimport lustre/attribute\nimport lustre/effect.{type Effect}\nimport lustre/element.{type Element}\nimport lustre/event\n\npub fn main() {\n  let app = lustre.application(init, update, view)\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n\n  Nil\n}\n\ntype Model\n\nfn init(_) -&gt; #(Model, Effect(Msg)) {\n  todo\n}\n\ntype Msg\n\nfn update(model: Model, msg: Msg) -&gt; #(Model, Effect(Msg)) {\n  todo\n}\n\nfn view(model: Model) -&gt; Element(Msg) {\n  todo\n}\n```\n\nThe `Model` for our grocery list will be a list of tuples containing the name\nof some produce and a quantity to purchase. For now, we&#39;ll initialise that with\njust an empty list:\n\n```gleam\ntype Model =\n  List(#(String, Int))\n\nfn init(_) -&gt; #(Model, Effect(Msg)) {\n  let model = []\n  let effect = effect.none()\n\n  #(model, effect)\n}\n```\n\nOur `Msg` type needs to cover changes to the grocery list, the user requesting\nto save the list, and the response from the server:\n\n```gleam\ntype Msg {\n  ServerSavedList(Result(Nil, String))\n  UserAddedProduct(name: String)\n  UserSavedList\n  UserUpdatedQuantity(name: String, amount: Int)\n}\n\nfn update(model: Model, msg: Msg) -&gt; #(Model, Effect(Msg)) {\n  case msg {\n    ServerSavedList(_) -&gt; todo\n    UserAddedProduct(_) -&gt; todo\n    UserSavedList -&gt; todo\n    UserUpdatedQuantity(_, _) -&gt; todo\n  }\n}\n```\n\nFinally, our `view` function will render the grocery list as an unordered list:\neach item will have an input field to update the quantity. Below the list will\nbe an input field to add a new item, and a button to save the list:\n\n```gleam\nfn view(model: Model) -&gt; Element(Msg) {\n  let styles = [\n    #(&quot;max-width&quot;, &quot;30ch&quot;),\n    #(&quot;margin&quot;, &quot;0 auto&quot;),\n    #(&quot;display&quot;, &quot;flex&quot;),\n    #(&quot;flex-direction&quot;, &quot;column&quot;),\n    #(&quot;gap&quot;, &quot;1em&quot;),\n  ]\n\n  html.div([attribute.style(styles)], [\n    view_grocery_list(model),\n    view_new_item(),\n    html.div([], [html.button([], [html.text(&quot;Sync&quot;)])]),\n  ])\n}\n\nfn view_new_item() -&gt; Element(Msg) {\n  let handle_click = fn(event) {\n    let path = [&quot;target&quot;, &quot;previousElementSibling&quot;, &quot;value&quot;]\n\n    event\n    |&gt; decipher.at(path, dynamic.string)\n    |&gt; result.map(UserAddedProduct)\n  }\n\n  html.div([], [\n    html.input([]),\n    html.button([event.on(&quot;click&quot;, handle_click)], [html.text(&quot;Add&quot;)]),\n  ])\n}\n\nfn view_grocery_list(model: Model) -&gt; Element(Msg) {\n  let styles = [#(&quot;display&quot;, &quot;flex&quot;), #(&quot;flex-direction&quot;, &quot;column-reverse&quot;)]\n\n  element.keyed(html.div([attribute.style(styles)], _), {\n    use #(name, quantity) &lt;- list.map(model)\n    let item = view_grocery_item(name, quantity)\n\n    #(name, item)\n  })\n}\n\nfn view_grocery_item(name: String, quantity: Int) -&gt; Element(Msg) {\n  let handle_input = fn(e) {\n    event.value(e)\n    |&gt; result.nil_error\n    |&gt; result.then(int.parse)\n    |&gt; result.map(UserUpdatedQuantity(name, _))\n    |&gt; result.replace_error([])\n  }\n\n  html.div([attribute.style([#(&quot;display&quot;, &quot;flex&quot;), #(&quot;gap&quot;, &quot;1em&quot;)])], [\n    html.span([attribute.style([#(&quot;flex&quot;, &quot;1&quot;)])], [html.text(name)]),\n    html.input([\n      attribute.style([#(&quot;width&quot;, &quot;4em&quot;)]),\n      attribute.type_(&quot;number&quot;),\n      attribute.value(int.to_string(quantity)),\n      attribute.min(&quot;0&quot;),\n      event.on(&quot;input&quot;, handle_input),\n    ]),\n  ])\n}\n```\n\n## Building for production\n\nWhen we&#39;re ready to deploy our application we will want to make sure our client\napp is _minified_. Minification is a process JavaScript build tools go through to\nrename variables, strip whitespace, and transform the code in other ways to make\nthe file smaller.\n\nLustre&#39;s build tools can produce minified JavaScript (and CSS, if you&#39;re using\nTailwind) bundles by providing the `--minify` flag:\n\n```sh\ngleam run -m lustre/dev build --minify --outdir=../server/priv/static\n```\n\nTypically, building a Lustre application will place the bundled JavaScript in the\nproject&#39;s `priv/static` directory. Because we&#39;re building a full stack application\nwe want our server to serve the JavaScript itself. The `--outdir` flag tells the\nbuild tool to place the output in our _server&#39;s_ priv directory instead.\n\nNext, in our server we can use Wisp&#39;s [`server_static`](https://hexdocs.pm/wisp/wisp.html#serve_static)\nmiddleware to automatically serve files from our server&#39;s `priv/static` directory.\nTo do that we&#39;ll modify our `handler` function:\n\n```gleam\nfn handler(req: wisp.Request) -&gt; wisp.Response {\n  let assert Ok(priv) = wisp.priv_directory(&quot;app&quot;)\n  let static_dir = priv &lt;&gt; &quot;/static&quot;\n  use &lt;- wisp.serve_static(req, under: &quot;/static&quot;, from: static_dir)\n  ...\n}\n```\n","url":"guide/06-full-stack-applications.html"},{"doc":"lustre","title":"lustre","content":"","url":"#"},{"doc":"lustre","title":"lustre","content":"&gt; **Note**: this guide is a work in progress and is not currently complete. Content\n&gt; here will change and be added over time. In the meantime, you can check out the\n&gt; [Gleam Discord server](https://discord.gg/Fm8Pwmy) if you have any questions about\n&gt; creating components with Lustre.\n\n\n# 06 Components\n\nIn the previous chapters of this guide we have explored the Model-View-Update\narchitecture and stressed the importance of having a _single source of truth_ in\nyour applications. Things are not always as simple in the real world, and there\nis a time for _encapsulated_ state. For those times, Lustre has components that\nallow you to nest self-contained MVU applications without leaking their implementation.\n\n## The best component is no component\n\nBefore we dive into how components work in Lustre, it&#39;s important to explore the\noptions we have to _avoid_ using them. Components require a little bit of set up\nand are conceptually &quot;heavier&quot; than components in frameworks like React or Svelte.\nIf you can avoid using components, you should, so let&#39;s take a look at how.\n\n### View functions\n\nIn Lustre, we call any function that returns an `Element` a &quot;view function&quot;. This\nmight seem obvious, but you can get quite far only using view functions and passing\nin state and update functions where needed. For the simplest cases, this could be\ndirect arguments to the view function:\n\n```gleam\npub fn my_button(colour: Colour, label: String, msg: msg) -&gt; Element(msg) {\n  html.button([colour.to_style(colour), event.on_click(msg)], [\n    html.text(label)\n  ])\n}\n```\n\nAs you build more complex UIs, you might find the number of arguments your view\nfunctions accept grows. A popular approach is to group those arguments into a\nrecord so that all the arguments to a view function can be treated as a single\npiece of data:\n\n```gleam\npub type MyButtonProps(msg) {\n  MyButtonProps(label: String, colour: Colour, msg: msg)\n}\n\npub fn my_button(props: MyButtonProps) -&gt; Element(msg) {\n  html.button([colour.to_style(props.colour), event.on_click(props.msg)], [\n    html.text(props.label)\n  ])\n}\n```\n\nThis approach lends itself well to the _builder pattern_. When we have many props\nthat are optional, or a highly customisable element, we can create builder functions\nto construct the props sequentially:\n\n```gleam\npub type MyButtonProps(msg) {\n  MyButtonProps(\n    label: String,\n    msg: msg,\n    colour: Colour,\n    variant: MyButtonVariant,\n    icon: Option(Icon)\n  )\n}\n\npub type MyButtonVariant {\n  Solid\n  Outline\n}\n\npub fn new(label: String, msg: msg) -&gt; MyButtonProps(msg) {\n  MyButtonProps(label, msg, Primary, Solid,  None)\n}\n\npub fn with_variant(\n  variant: MyButtonVariant,\n  props: MyButtonProps(msg)\n) -&gt; MyButtonProps(msg) {\n  MyButtonProps(props.label, props.msg, props.colour, variant, props.icon)\n}\n\npub fn with_icon(\n  icon: Icon,\n  props: MyButtonProps(msg)\n) -&gt; MyButtonProps(msg) {\n  MyButtonProps(props.label, props.msg, props.colour, props.variant, Some(icon))\n}\n\n// ...\n\npub fn view(props: MyButtonProps(msg)) -&gt; Element(msg) {\n  let variant = case props.variant {\n    Solid -&gt; attribute.class(&quot;solid&quot;)\n    Outline -&gt; attribute.class(&quot;outline&quot;)\n  }\n\n  let icon = case props.icon {\n    Some(icon) -&gt; icon.view(icon)\n    None -&gt; html.none()\n  }\n\n  html.button(\n    [\n      colour.to_style(props.colour),\n      variant,\n      event.on_click(props.msg)\n    ],\n    [\n      icon,\n      html.text(props.label)\n    ]\n  )\n}\n```\n\nSome folks like to _really_ lean into the builder pattern. If you want to try\nsomething fancy, you can look into the [phantom builder pattern](https://www.youtube.com/watch?v=3lYHFctx2Ks)\nto see how you can use the type system to enforce things like required props or\nprevent setting the same prop twice. We use this approach in [lustre/ssg](https://hexdocs.pm/lustre_ssg)!\n\n### Nested MVU\n","url":"guide/08-components.html"},{"doc":"lustre","title":"lustre","content":"&gt; **Note**: this guide is a work in progress and is not currently complete. Content\n&gt; here will change and be added over time. In the meantime, you can check out the\n&gt; [Gleam Discord server](https://discord.gg/Fm8Pwmy) if you have any questions about\n&gt; Lustre&#39;s server components.\n\n# 09 Server Components\n\nPlease see the following resources:\n- the [api reference](https://hexdocs.pm/lustre/lustre/server_component.html) for server components.\n- an example of how to set up [server components using mist](https://github.com/lustre-labs/lustre/tree/main/examples/99-server-components).\n- a repo showcasing the same [lustre application used as a SPA, a web component, and as a server component](https://github.com/hayleigh-dot-dev/lustre-universal-component-demo/tree/main).\n","url":"guide/09-server-components.html"},{"doc":"lustre","title":"lustre","content":"","url":"#"},{"doc":"lustre","title":"lustre","content":"# 04 SPA deployments\n\nIf you have built a Lustre application that doesn&#39;t have a backend you need to deploy, you can use a number of static hosting services to get your SPA online. In this guide we&#39;ll look at two options, GitHub Pages and Cloudflare Pages, but the principles can be applied to other services as well.\n\nIf you are planning on building and deploying a full stack Gleam project, you probably don&#39;t want to follow this guide!\n\n## Prerequisites\n\nIn this guide we assume that you can build your Lustre SPA with `lustre_dev_tools`:\n\n```\ngleam run -m lustre/dev build app --minify\n```\n\nand that this results in an application in your `priv/static` directory in your repository root directory. The built application is then loaded in a `&lt;script&gt;` tag in an HTML file `index.html` located in your repository root directory.\n\nIn other words, this project setup closely resembles the [Hello World example](https://github.com/lustre-labs/lustre/tree/main/examples/01-hello-world).\n\n&gt; **Note**: when using the `--minify` flag in the build command, we need to update the `&lt;script&gt;` tag in `index.html` to point to the minified file. In the Hello World example, this would be `/priv/static/app.min.mjs` instead of `/priv/static/app.mjs`. In the GitHub Action workflows below we have automated this step with a `sed`-command. In this command, we also assumed that your app is called `app` just like in the Hello World example, so if your app has a different name you need to update the command accordingly.\n\nFinally, we also assume that you have a GitHub repository set up for your Lustre application.\n\n## Deploying with GitHub Pages\n\nThere are multiple ways to deploy a Lustre SPA with GitHub Pages. For this guide we will focus on automated deployments using GitHub Actions. We will build our application with `lustre_dev_tools` and use the GitHub Actions workflow to deploy our application.\n\n&gt; **Note**: when deploying with GitHub Pages, remember that your application will be served at `https://&lt;username&gt;.github.io/&lt;repository&gt;`. The `&lt;repository&gt;` part of the URL is important as you will need to update the `&lt;script&gt;` tag in `index.html` to point to the correct file. In the Hello World example, this would be `/&lt;repository&gt;/priv/static/app.mjs`. Otherwise, your application will not load and you will see a 404 error in the browser console.\n\n### Setting up GitHub Pages\n\n1. Go to your repository settings on GitHub.\n2. Navigate to the Pages section.\n3. Set the source to GitHub Actions.\n\n### Setting up GitHub Actions\n\n#### Create a GitHub Actions Workflow\n\nCreate a new file in your repository at `.github/workflows/deploy.yml` and use the following template as a starting point:\n\n```yaml\nname: Deploy to GitHub Pages\n\non:\n  push:\n    branches:\n      - main\n\njobs:\n  deploy:\n    name: Deploy to GitHub Pages\n    runs-on: ubuntu-latest\n    permissions:\n      contents: read\n      deployments: write\n      pages: write\n      id-token: write\n    steps:\n      - name: Checkout repo\n        uses: actions/checkout@v4\n      - name: Set up Gleam\n        uses: erlef/setup-beam@v1\n        with:\n          otp-version: &quot;27.0&quot;\n          gleam-version: &quot;1.4.1&quot;\n          rebar3-version: &quot;3&quot;\n      - name: Install dependencies\n        run: gleam deps download\n      - name: Build app\n        run: gleam run -m lustre/dev build app --minify\n      - name: Copy output to dist\n        run: |\n          mkdir -p dist\n          cp index.html dist/index.html\n          cp -r priv dist/priv\n      - name: Update path in index.html to use minified app\n        run: sed -i &#39;s|priv/static/app.mjs|priv/static/app.min.mjs|&#39; dist/index.html\n      - name: Setup Pages\n        uses: actions/configure-pages@v5\n      - name: Upload artifact\n        uses: actions/upload-pages-artifact@v3\n        with:\n          path: &quot;dist&quot;\n      - name: Deploy to GitHub Pages\n        id: deployment\n        uses: actions/deploy-pages@v4\n```\n\n&gt; **Note**: Make sure to update the Gleam and OTP versions to match your project&#39;s requirements. Also verify that the app name used in the `sed`-command matches the name of your application.\n\nThis workflow:\n\n- Installs Gleam and its dependencies.\n- Builds the Lustre application.\n- Copies the built application to a `dist/` directory.\n- Uploads the `dist/` directory as an artifact.\n- Deploys the artifact to GitHub Pages.\n\n### Deploying\n\nAfter setting up the GitHub Actions workflow, you can now push your changes to the `main` branch to trigger a deployment to GitHub Pages.\n\nYour application should now be available at the GitHub Pages URL, which is usually `https://&lt;username&gt;.github.io/&lt;repository&gt;`.\n\n## Deploying with Cloudflare Pages\n\nThere are multiple ways to deploy a Lustre SPA with Cloudflare Pages. For this guide we will focus on automated deployments using GitHub Actions. We will opt out of Cloudflare&#39;s &quot;Automatic Deployments&quot; since it does not support Gleam and/or Lustre. Instead, we build our application with `lustre_dev_tools` (similar to the GitHub pages guide above) and use the Wrangler action to deploy our application.\n\n### Setting up Cloudflare Pages\n\n#### Create a Cloudflare Pages Project\n\n1. Go to the Cloudflare dashboard.\n2. Navigate to the &quot;Workers &amp; Pages&quot; section.\n3. Create a new project and connect it to your GitHub repository.\n4. During setup, select the appropriate branch (e.g. `main`) to deploy from.\n5. Ignore the build settings for now.\n\n#### Disable Automatic Deployments\n\n1. After creating the project, navigate to the project settings.\n2. Under &quot;Builds &amp; Deployments&quot;, disable &quot;Automatic Deployments&quot; for production and preview branches.\n\n### Setting up GitHub Actions\n\n#### Set Environment Variables\n\n1. Go to your repository settings on GitHub.\n2. Navigate to the Secrets section.\n3. Add a new `CLOUDFLARE_ACCOUNT_ID` secret based on:\n   - Go to any Zone in your Cloudflare account.\n   - Copy the Account ID from the right sidebar and paste it as the secret value.\n4. Add a new `CLOUDFLARE_API_TOKEN` secret based on:\n   - Go to &quot;My Profile&quot; from the top right dropdown menu.\n   - Navigate to the &quot;API Tokens&quot; section.\n   - Create a new custom API token with the `Cloudflare Pages: Edit` permission.\n   - Copy the API token and paste it as the secret value.\n\n#### Create a GitHub Actions Workflow\n\nCreate a new file in your repository at `.github/workflows/deploy.yml` and use the following template as a starting point:\n\n```yaml\nname: Deploy to Cloudflare Pages\n\non:\n  push:\n    branches:\n      - main\n\njobs:\n  deploy:\n    name: Deploy to Cloudflare Pages\n    runs-on: ubuntu-latest\n    permissions:\n      contents: read\n      deployments: write\n    steps:\n      - name: Checkout repo\n        uses: actions/checkout@v4\n      - name: Set up Gleam\n        uses: erlef/setup-beam@v1\n        with:\n          otp-version: &quot;27.0&quot;\n          gleam-version: &quot;1.4.1&quot;\n          rebar3-version: &quot;3&quot;\n      - name: Install dependencies\n        run: gleam deps download\n      - name: Build app\n        run: gleam run -m lustre/dev build app --minify\n      - name: Copy output to dist\n        run: |\n          mkdir -p dist\n          cp index.html dist/index.html\n          cp -r priv dist/priv\n      - name: Update path in index.html to use minified app\n        run: sed -i &#39;s|priv/static/app.mjs|priv/static/app.min.mjs|&#39; dist/index.html\n      - name: Deploy with Wrangler\n        uses: cloudflare/wrangler-action@v3\n        with:\n          apiToken: ${{ secrets.CLOUDFLARE_API_TOKEN }}\n          accountId: ${{ secrets.CLOUDFLARE_ACCOUNT_ID }}\n          command: pages deploy dist --project-name &lt;YOUR_PROJECT_NAME&gt;\n```\n\n&gt; **Note**: Make sure to replace `&lt;YOUR_PROJECT_NAME&gt;` with the name of your Cloudflare Pages project and also to update the Gleam and OTP versions to match your project&#39;s requirements. Also verify that the app name used in the `sed`-command matches the name of your application.\n\nThis workflow:\n\n- Installs Gleam and its dependencies.\n- Builds the Lustre application.\n- Copies the built application to a `dist/` directory.\n- Deploys the `dist/` directory to Cloudflare Pages using Wrangler.\n\n### Deploying\n\nAfter setting up the GitHub Actions workflow, you can now push your changes to the `main` branch to trigger a deployment to Cloudflare Pages.\n\nYour application should now be available at the Cloudflare Pages URL, which is usually `https://&lt;project-name&gt;.pages.dev`.\n","url":"guide/04-spa-deployments.html"},{"doc":"lustre","title":"lustre","content":"# 07 Full-Stack Deployments\n\nIf you have built a full-stack Lustre application then you can use Docker and a platform provider such as [Fly.io](https://fly.io/) to deploy your application.\n\nIn this guide, you will learn how to manually deploy a Lustre full-stack application to Fly.io using the `flyctl` CLI. You will also learn how to leverage GitHub Actions for continuous deployments.\n\nIf you are planning on deploying a Lustre SPA without a backend, you probably don&#39;t want to follow this guide! Instead, refer to the [Lustre SPA deployment guide](./04-spa-deployments.html).\n\n## Prerequisites\n\nIn this guide, we assume that you have a Lustre full-stack application with the following _monorepo_ structure consisting of three separate Gleam projects:\n\n```sh\ncommon/\nclient/\nserver/\n```\n\nHere, the `common/` directory contains the Lustre application logic which is agnostic to the platform it runs on.\n\nThe `client/` directory contains the client-side code, which is typically only responsible for the boilerplate necessary to run the Lustre application in the browser. The client-side code uses the JavaScript target and is built with `lustre_dev_tools` with the `--outdir` flag set to `../server/priv/static`.\n\nThe `server/` directory contains the server-side code and runs the same Lustre application as a server component. The server-side code uses the Erlang target.\n\nIn this guide, we assume this monorepo structure because it happens to be the same structure used in the [Lustre full-stack application guide](./06-full-stack-applications.html).\n\nIf your full-stack application is structured in a slightly different way, don&#39;t worry! You may need to adjust the [`Dockerfile`](#dockerfile) but otherwise the other sections in this guide should still be applicable.\n\n### `flyctl`\n\nTo generate a `fly.toml` configuration file and to manually deploy your application to Fly.io, you will need the `flyctl` CLI installed on your local machine. Please refer to the [`flyctl` installation section in the docs](https://fly.io/docs/getting-started/installing-flyctl/).\n\n### Dockerfile\n\nUse this Dockerfile as a starting point for your Lustre full-stack application, regardless if you are planning to use manual deployments or continuous deployments with GitHub Actions.\n\nIn the Dockerfile, we compile both the client and server code in a separate build stage and then only copy what is needed to the final image. This reduces the final image size and ensures that only the necessary files are included.\n\n```Dockerfile\nARG GLEAM_VERSION=v1.4.1\n\nFROM ghcr.io/gleam-lang/gleam:${GLEAM_VERSION}-erlang-alpine AS builder\n\n# Add project code\nCOPY ./common /build/common\nCOPY ./client /build/client\nCOPY ./server /build/server\n\n# Compile the client code\nRUN cd /build/client \\\n  &amp;&amp; gleam run -m lustre/dev build app --outdir=/build/server/priv/static\n\n# Compile the server code\nRUN cd /build/server \\\n  &amp;&amp; gleam export erlang-shipment\n\n# Start from a clean slate\nFROM ghcr.io/gleam-lang/gleam:${GLEAM_VERSION}-erlang-alpine\n\n# Copy the compiled server code from the builder stage\nCOPY --from=builder /build/server/build/erlang-shipment /app\n\n# Run the server\nWORKDIR /app\nENTRYPOINT [&quot;/app/entrypoint.sh&quot;]\nCMD [&quot;run&quot;]\n```\n\n&gt; **Note**: Make sure to set the `GLEAM_VERSION` to match your project&#39;s requirements.\n\n## Manual Deployment\n\nIn this section, we will learn how to manually deploy a Lustre full-stack application to Fly.io using the `flyctl` CLI.\n\n### Setting Up `fly.toml`\n\nTo deploy your Lustre full-stack application to Fly.io, you will need to create a `fly.toml` configuration file in the root directory of your project. You can create a `fly.toml` file by running the following command:\n\n```bash\nfly launch --no-deploy\n```\n\nWhen prompted, you will need to name your app and select a region. Confirm the settings after making your selections.\n\n### Deploying\n\nOnce you have set up the `fly.toml` configuration file and added the `Dockerfile` to your project, you can now build and deploy your application to Fly.io by running the following command:\n\n```bash\nflyctl deploy\n```\n\nThe `flyctl deploy` command will build a Docker image of your application and deploy it to Fly.io. The output will include a URL where you can access your application.\n\n## Automated Deployments\n\nIn this section, we will learn how to set up automated deployments for a Lustre full-stack application using GitHub Actions.\n\n### Setting Up `fly.toml`\n\nIf you did not set up a `fly.toml` configuration file during the manual deployment process, you can create one by running the following command in your project root directory:\n\n```bash\nfly launch --no-deploy\n```\n\nWhen prompted, you will need to name your app and select a region. Confirm the settings after making your selections.\n\n### Setting Up GitHub Actions\n\nTo enable automated deployments you will need a Fly API token. Navigate to the project root directory and run the following command:\n\n```bash\nfly tokens create deploy -x 999999h\n```\n\nCopy the entire output, including the FlyV1 prefix, and add the Fly API token to your GitHub repository as a repository secret.\n\n1. Go to your repository on GitHub.\n2. Navigate to &quot;Settings&quot; &gt; &quot;Secrets and variables&quot; &gt; &quot;Actions&quot;.\n3. Create a new repository secret named `FLY_API_TOKEN` and paste the Fly API token as the value.\n\nWith the API token in place, create a new file in your repository at `.github/workflows/fly.yml` and use the following template as a starting point:\n\n```yaml\nname: Deploy to Fly.io\non:\n  push:\n    branches:\n      - main\n\njobs:\n  deploy:\n    name: Deploy to Fly.io\n    runs-on: ubuntu-latest\n    concurrency: deploy-group\n    steps:\n      - uses: actions/checkout@v4\n      - uses: superfly/flyctl-actions/setup-flyctl@master\n      - run: flyctl deploy --remote-only\n        env:\n          FLY_API_TOKEN: ${{ secrets.FLY_API_TOKEN }}\n\n```\n\nThis workflow:\n\n- Checks out your repository code.\n- Sets up the Flyctl CLI tool.\n- Deploys the application to Fly.io whenever changes are pushed to the main branch.\n\n\n### Deploying\n\nYou should now have a fully automated deployment pipeline for your Lustre full-stack application on Fly.io.\n\nWhenever you push changes to your repository, your application will be automatically rebuilt and redeployed.\n","url":"guide/07-full-stack-deployments.html"},{"doc":"lustre","title":"lustre","content":"","url":"#"},{"doc":"lustre","title":"lustre","content":"# Lustre for Elm developers\n\nLustre has been directly inspired by Elm and shares some of the primary architectural\nfeatures of &quot;The Elm Architecture&quot;, otherwise known as Model-View-Update. This\nguide is for Elm developers who are new to Lustreand want to get up to speed quickly.\n\n## How do I...?\n\n### Setup a new project\n\n**In Elm**, all you really need to get started is to install the `elm` binary.\nRunning `elm make` against an Elm file will transpile your code to either Javascript,\nor HTML with the Javascript output inlined. Most people will build out their own\ntoolchain to support build-on-save and hot-reload, with tools like Vite or Webpack\nwith the appropriate plugins. A simple hello world might look like this:\n\n```elm\nmodule Main exposing (main)\n\nimport Html\n\nmain =\n    Html.text &quot;Hello, world&quot;\n\n```\n\n**In Lustre** you need to install the `lustre` package with `gleam add lustre`.\nMost Lustre projects will add the dev tools too with `gleam add --dev lustre_dev_tools`.\nA simple hello world might look like this:\n\n```gleam\nimport lustre\nimport lustre/element/html\n\npub fn main() {\n  let app = lustre.element(html.h1([], [html.text(&quot;Hello, world&quot;)]))\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n}\n```\n\n### Render some HTML\n\n**In Elm**, you can call functions in the `elm/html` package to render HTML\nelements. The `Html` module in `elm/html` contains functions for most standard\nHTML tags; these functions take as parameters a list of attributes from\n`Html.Attributes`, or events from `Html.Events` - as well as a list of child\nelements:\n\n```elm\nHtml.button\n  [ Html.Attributes.class &quot;primary&quot;\n  , Html.Events.onClick ButtonClicked\n  ]\n  [ Html.text &quot;Click me&quot; ]\n```\n\n---\n\n**In Lustre**, HTML is rendered by calling functions, many of whom share the same\nsignature - functions in `lustre/element/html` represent HTML tags, and most\nfunctions accept a list of `lustre/attribute` or `lustre/event` values, as well\nas a list of child elements.\n\n```lustre\nhtml.button([attribute.class(&quot;primary&quot;), event.on_click(ButtonClicked)], [\n  html.text(&quot;Click me&quot;)\n])\n```\n\n### Render some text\n\n**In Elm**, text is rendered by passing a `String` to the `Html.text` function:\n\n```elm\nHtml.span [] [ Html.text &lt;| &quot;Hello, &quot; ++ name ]\n```\n\n---\n\n**In Lustre**, text is rendered by passing a `String` to the `html.text` or\n`element.text` functions:\n\n```gleam\nhtml.span([], [\n  html.text(&quot;Hello, &quot; &lt;&gt; name),\n])\n```\n\n### Manage state\n\n**In Elm** all state is stored in a single `Model` type and updates happen through\na central `update` function:\n\n```elm\ntype alias Model = Int\n\ninit : Model\ninit = 0\n\ntype Msg\n    = Incr\n    | Decr\n\nupdate : Msg -&gt; Model -&gt; Model\nupdate msg model =\n    case msg of\n        Incr -&gt;\n            model + 1\n\n        Decr -&gt;\n            model - 1\n```\n\n---\n\n**In Lustre** all state is stored in a single `Model` type and updates happen\nthrough a central `update` function, just like in Elm:\n\n```gleam\nfn init(_) {\n  0\n}\n\ntype Msg {\n  Incr\n  Decr\n}\n\nfn update(model: Model, msg: Msg) -&gt; Msg {\n  case msg {\n    Incr -&gt; model + 1\n    Decr -&gt; model - 1\n  }\n}\n```\n\n### Handle events\n\n**In Elm** event handlers are decoders for event objects. When the decoder succeeds,\nthat value is dispatched as a message to your `update` function.\n\n```elm\nHtml.input [ Html.Events.onInput UserUpdatedNameField ] []\n\ntype Msg\n  = UserUpdatedNameField String\n\ntype alias Model = { name : String }\n\nupdate : Msg -&gt; Model -&gt; Model\nupdate msg model =\n  case msg of\n    UserUpdatedNameField name\n      { model | name = name }\n```\n\n---\n\n**In Lustre** event handlers work in the same way. Lustre provides functions to\nhandle most common events, in [`lustre/effect`](https://hexdocs.pm/lustre/lustre/effect.html):\n\n```gleam\nbutton([on_click(Decr)], [text(&quot;-&quot;)])\n```\n\n```gleam\ninput([on_input(UpdateInput)])\n```\n\n```gleam\ndiv([on(&quot;mousemove&quot;, fn(event) {\n  ...\n}], [...])\n```\n\n### Fetch data\n\n**In Elm** you can fetch data by making a HTTP request. HTTP request functions\nboth return a value of type `Cmd msg`, and are handled by the application&#39;s `update`\nfunction; the payload from the response is available within the `update` function\nand can be used to update the `Model`, or call other functions that return a value\nof type `Cmd msg`.\n\n```elm\ntype Msg\n  = ApiReturnedBookResponse (Result Http.Error String)\n\ngetBook : Cmd Msg\ngetBook =\n  Http.get\n    { url = &quot;https://elm-lang.org/assets/public-opinion.txt&quot;\n    , expect = Http.expectString ApiReturnedBookResponse\n    }\n\ntype alias Model = { bookResponse : Result Http.Error String }\n\nupdate : Msg -&gt; Model -&gt; ( Model, Cmd Msg )\nupdate msg model =\n  case msg of\n    ApiReturnedBookResponse response -&gt;\n      { model | bookResponse = response }\n```\n\n---\n\n**In Lustre**, the approach is similar, using types and functions from the\n[`lustre_http` package](https://hexdocs.pm/lustre_http/lustre_http.html):\n\n```gleam\npub type Msg {\n  ApiReturnedBookResponse(Result(String, lustre_http.HttpError))\n}\n\nfn get_book() -&gt; effect.Effect(Msg) {\n  lustre_http.get(\n    &quot;https://elm-lang.org/assets/public-opinion.txt&quot;,\n    lustre_http.expect_text(ApiReturnedBookResponse)\n  )\n}\n\npub type Model {\n  Model(book_response: Result(String, HttpError))\n}\n\n\npub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n  case msg {\n    ApiReturnedBookResponse(response) -&gt; #(Model(..model, book_response: response), effect.none())\n  }\n}\n\n```\n\n## Where to go next\n\nTo walk through setting up a new Lustre project and building your first app, check\nout the [quickstart guide](https://hexdocs.pm/lustre/guide/01-quickstart.html).\n\nIf you prefer to learn by example, we have a collection of examples that show\noff specific features and patterns in Lustre. You can find them in the\n[examples directory](https://hexdocs.pm/lustre/reference/examples.html)\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"cheatsheets/elm.html"},{"doc":"lustre","title":"lustre","content":"# Lustre for React developers\n\nIn some ways React and Lustre share the same DNA. But in a lot of other ways they\ncan be quite different! This guide is for React developers who are new to Lustre\nand want to get up to speed quickly.\n\n## How do I...?\n\n### Setup a new project\n\n**In React** you are encouraged to use a meta framework like Next.js or Remix. To\nstart a barebones project you need to run `npm install react react-dom`. You will\ntypically use a bundler that can transpile JSX like `npm install --save-dev vite`.\nMany modern projects use TypeScript as well: `npm install --save-dev typescript`.\nA simple hello world might look like this:\n\n```jsx\n// src/index.js\nimport { createRoot } from &quot;react-dom/client&quot;;\n\nconst root = createRoot(document.getElementById(&quot;app&quot;));\n\nroot.render(&lt;h1&gt;Hello, world&lt;/h1&gt;);\n```\n\nTo run the project you could use Vite&#39;s development server with `npx vite`.\n\n**In Lustre** you need to install the `lustre` package with `gleam add lustre`.\nMost Lustre projects will add the dev tools too with `gleam add --dev lustre_dev_tools`.\nA simple hello world might look like this:\n\n```gleam\n// main.gleam\nimport lustre\nimport lustre/element/html\n\npub fn main() {\n  let app = lustre.element(html.h1([], [html.text(&quot;Hello, world&quot;)]))\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n}\n```\n\n### Render some HTML\n\n**In React** you can use JSX to render HTML elements. JSX is a syntax extension\nfor JavaScript that looks like HTML that lets you interpolate JavaScript expressions\ninto your markup. Here&#39;s an example:\n\n```jsx\n&lt;button className=&quot;primary&quot;&gt;Click me&lt;/button&gt;\n```\n\n**In Lustre** HTML is rendered by calling functions (this is what JSX compiles to\nas well).\n\n```gleam\nbutton([class(&quot;primary&quot;)], [text(&quot;Click me&quot;)])\n```\n\n### Render some text\n\n**In React** a string is a valid type of React node, so you can render text by\njust writing it in your JSX:\n\n```jsx\nHello;\n```\n\nTo concatenate text with other variables or expressions, you can use curly braces:\n\n```jsx\nHello {name}\n```\n\n**In Lustre** because of Gleam&#39;s type system, all elements must be Lustre&#39;s `Element`\ntype. To render text you need to use the `text` function:\n\n```gleam\ntext(&quot;Hello&quot;)\ntext(&quot;Hello&quot; &lt;&gt; name)\n```\n\n### Manage state\n\n**In React**...\n\n**In Lustre**...\n\n### Handle events\n\n**In React**...\n\n**In Lustre**...\n\n### Write a component\n\n**In React**...\n\n**In Lustre**...\n\n### Fetch data\n\n**In React**...\n\n**In Lustre**...\n\n## Where to go next\n\nTo walk through setting up a new Lustre project and building your first app, check\nout the [quickstart guide](https://hexdocs.pm/lustre/guide/01-quickstart.html).\n\nIf you prefer to learn by example, we have a collection of examples that show\noff specific features and patterns in Lustre. You can find them in the\n[examples directory](https://hexdocs.pm/lustre/reference/examples.html)\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"cheatsheets/react.html"},{"doc":"lustre","title":"lustre","content":"# Lustre for LiveView developers\n\nComing from LiveView, many things about Lustre will feel very familiar. But in many\nother ways it will be quite different. This guide is for LiveView developers who\nare new to Lustre and want to get up to speed quickly.\n\n## How do I...?\n\n### Setup a new project\n\n**In LiveView** you create a new Phoenix project with `mix phx.new`. A simple\nhello world might look like this:\n\n```elixir\n # lib/my_app_web/live/hello_live.ex\n defmodule MyAppWeb.HelloLive do\n  use MyAppWeb, :live_view\n\n  def render(assigns) do\n    ~H&quot;&quot;&quot;\n    &lt;h1&gt;Hello, Joe&lt;/h1&gt;\n    &quot;&quot;&quot;\n  end\n\n  def mount(_params, _session, socket) do\n    {:ok, socket}\n  end\n end\n```\n\nTo start your dev server, run `mix phx.start`\n\n---\n\n**In Lustre**, after you&#39;ve created a new Gleam project with `gleam new`, you need\nto install the `lustre` package with `gleam add lustre`. Most Lustre projects will\ndd the dev tools too with `gleam add --dev lustre_dev_tools`. A simple hello world\nmight look like this:\n\n```gleam\n// main.gleam\nimport lustre\nimport lustre/element/html\n\npub fn main() {\n  let app = lustre.element(html.h1([], [html.text(&quot;Hello, Joe&quot;)]))\n  let assert Ok(_) = lustre.start(app, &quot;#app&quot;, Nil)\n}\n```\n\nTo start your dev server, run `gleam run -m lustre/dev start`\n\n### Render some HTML\n\n**In LiveView** you use HEEx templates to render HTML elements. This looks like\nHTML and allows you to interpolate Elixir code and dynamic values in your template.\n\n```html\n&lt;button class=&quot;primary&quot;&gt;Click me&lt;/button&gt;\n```\n\n---\n\n**In Lustre** HTML is rendered by calling functions. Usually the first argument\nis a list of `Attribute` types, and the second argument a list of `Element` type\nchildren.\n\n```gleam\nbutton([class(&quot;primary&quot;)], [text(&quot;Click me&quot;)])\n```\n\n### Render some text\n\n**In LiveView** a string is a valid type of node, so you can render text by just\nwriting it in your HEEx template:\n\n```html\n&lt;span&gt;Hello&lt;/span&gt;\n```\n\nTo concatenate text with other variables or expressions, you can use `&lt;%= %&gt;` tags.\n\n```html\n&lt;span&gt;Hello &lt;%= @name %&gt;&lt;/span&gt;\n```\n\n---\n\n**In Lustre** because of Gleam&#39;s type system, all elements must be Lustre&#39;s `Element`\ntype. To render text you need to use the `text` function:\n\n```gleam\nspan([], [\n  text(&quot;Hello&quot;),\n  text(&quot;Hello&quot; &lt;&gt; name),\n])\n```\n\n### Manage state\n\n**In LiveView** all your state is stored in the `assigns` key of the socket. Updates\nhappen by sending events to the LiveView process, where the event handlers will\nupdate the relevant state and return the updated socket.\n\n```elixir\ndef mount(_params, _session, socket) do\n  socket = assign(socket, :value, 1)\n  {:ok, socket}\nend\n\ndef handle_event(&quot;increment&quot;, _params, socket) do\n  {:noreply, update(socket, :value, &amp;(&amp;1 + 1))}\nend\n\ndef handle_event(&quot;decrement&quot;, _params, socket) do\n  {:noreply, update(socket, :value, &amp;(&amp;1 - 1))}\nend\n```\n\n---\n\n**In Lustre** all state is stored in a single `Model` type and updates happen\nthrough a central `update` function. A notable difference here is that Gleam does\nnot allow pattern matching in function heads, so instead you use a `case` statement\nto match on the different messages and handle them accordingly.\n\n```gleam\nfn init(_) {\n  0\n}\n\ntype Msg {\n  Incr\n  Decr\n}\n\nfn update(model, msg) {\n  case msg {\n    Incr -&gt; model + 1\n    Decr -&gt; model - 1\n  }\n}\n```\n\n### Handle events\n\n**In LiveView** you can bind messages to events in your HEEx template and define\n`handle_event/3` functions that receive the event name, any addtional parameters\nand the socket. These usually change some kind of state, and return the updated\nsocket. LiveView provides handlers for most common events and passes the accompanying\nvalues to your event handler.\n\n```html\n&lt;button phx-click=&quot;decrement&quot;&gt;-&lt;/button&gt;\n```\n\n```html\n&lt;input phx-change=&quot;update_input&quot;&gt; /&gt;\n```\n\nWith a bit of JavaScript you can also handle events that are not provided in LiveView.\n\n```html\n&lt;div id=&quot;my-div&quot; phx-hook=&quot;MouseMove&quot;&gt;&lt;/div&gt;\n```\n\n```js\nlet Hooks = {};\n\nHooks.MouseMove = {\n  mounted() {\n    this.el.addEventListener(&quot;mousemove&quot;, e =&gt; {\n      ...\n    })\n  }\n}\n```\n\n---\n\n**In Lustre** event handlers are decoders for event objects. When the decoders\nsucceeds, that value is passed to your `update` function. Lustre provides functions\nto handle most common events.\n\n```gleam\nbutton([on_click(Decr)], [text(&quot;-&quot;)])\n```\n\n```gleam\ninput([on_input(UpdateInput)])\n```\n\n```gleam\ndiv([on(&quot;mousemove&quot;, fn(event) {\n  ...\n}], [...])\n```\n\n### Write a component\n\n**In LiveView** you can define function components to organise and reuse markup.\n\n```elixir\ndef render(assigns) do\n  ~H&quot;&quot;&quot;\n    &lt;.greet name={@name} /&gt;\n  &quot;&quot;&quot;\nend\n\ndefp greet(assigns) do\n  ~H&quot;&quot;&quot;\n    &lt;h1&gt;Hello, &lt;%= @name %&gt;&lt;/h1&gt;\n  &quot;&quot;&quot;\nend\n```\n\nLiveView also allows for live components that contain state, markup and events,\nbut Lustre does not have the equivalent of these.\n\n---\n\n**In Lustre** components are more commonly referred to as &quot;view functions&quot;. They\nare regular Gleam functions.\n\n```gleam\nfn greet(name: String) -&gt; Element(Msg) {\n  html.h1([], [html.text(&quot;Hello, &quot;  &lt;&gt; name)])\n}\n```\n\n---\n\n### Fetch data\n\n**In LiveView** ...\n\n---\n\n**In Lustre**...\n\n## Where to go next\n\nTo walk through setting up a new Lustre project and building your first app, check\nout the [quickstart guide](https://hexdocs.pm/lustre/guide/01-quickstart.html).\n\nIf you prefer to learn by example, we have a collection of examples that show\noff specific features and patterns in Lustre. You can find them in the\n[examples directory](https://hexdocs.pm/lustre/reference/examples.html)\n\nIf you&#39;re having trouble with Lustre or not sure what the right way to do\nsomething is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\nYou could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"cheatsheets/liveview.html"},{"doc":"lustre/effect","title":"Effect","content":"pub opaque type Effect(msg)\n The `Effect` type treats side effects as data and is a way of saying &quot;Hey\n Lustre, do this thing for me.&quot; Each effect specifies two things:\n\n 1. The side effects for the runtime to perform.\n\n 2. The type of messages that (might) be sent back to the program in response.\n\n\n\n\nSynonyms:\nlustre/effect.Effect\nlustre/effect Effect","url":"lustre/effect.html#Effect"},{"doc":"lustre/effect","title":"batch","content":"pub fn batch(effects: List(Effect(a))) -&gt; Effect(a)\n Batch multiple effects to be performed at the same time.\n\n **Note**: The runtime makes no guarantees about the order on which effects\n are performed! If you need to chain or sequence effects together, you have\n two broad options:\n\n 1. Create variants of your `msg` type to represent each step in the sequence\n    and fire off the next effect in response to the previous one.\n\n 2. If you&#39;re defining effects yourself, consider whether or not you can handle\n    the sequencing inside the effect itself.\n\nSynonyms:\nlustre/effect.batch\nlustre/effect batch","url":"lustre/effect.html#batch"},{"doc":"lustre/effect","title":"from","content":"pub fn from(effect: fn(fn(a) -&gt; Nil) -&gt; Nil) -&gt; Effect(a)\n Construct your own reusable effect from a custom callback. This callback is\n called with a `dispatch` function you can use to send messages back to your\n application&#39;s `update` function.\n\n Example using the `window` module from the `plinth` library to dispatch a\n message on the browser window object&#39;s `&quot;visibilitychange&quot;` event.\n\n \n import lustre/effect.{type Effect}\n import plinth/browser/window\n\n type Model {\n   Model(Int)\n }\n\n type Msg {\n   FetchState\n }\n\n fn init(_flags) -&gt; #(Model, Effect(Msg)) {\n   #(\n     Model(0),\n     effect.from(fn(dispatch) {\n       window.add_event_listener(&quot;visibilitychange&quot;, fn(_event) {\n         dispatch(FetchState)\n       })\n     }),\n   )\n }\n \nSynonyms:\nlustre/effect.from\nlustre/effect from","url":"lustre/effect.html#from"},{"doc":"lustre/effect","title":"map","content":"pub fn map(effect: Effect(a), f: fn(a) -&gt; b) -&gt; Effect(b)\n Transform the result of an effect. This is useful for mapping over effects\n produced by other libraries or modules.\n\n **Note**: Remember that effects are not _required_ to dispatch any messages.\n Your mapping function may never be called!\n\nSynonyms:\nlustre/effect.map\nlustre/effect map","url":"lustre/effect.html#map"},{"doc":"lustre/effect","title":"none","content":"pub fn none() -&gt; Effect(a)\n Most Lustre applications need to return a tuple of `#(model, Effect(msg))`\n from their `init` and `update` functions. If you don&#39;t want to perform any\n side effects, you can use `none` to tell the runtime there&#39;s no work to do.\n\nSynonyms:\nlustre/effect.none\nlustre/effect none","url":"lustre/effect.html#none"},{"doc":"lustre/effect","title":"lustre/effect","content":" In other frameworks it&#39;s common for components to perform side effects\n whenever the need them. An event handler might make an HTTP request, or a\n component might reach into the DOM to focus an input.\n\n In Lustre we try to keep side effects separate from our main program loop.\n This comes with a whole bunch of benefits like making it easier to test and\n reason about our code, making it possible to implement time-travel debugging,\n or even to run our app on the server using Lustre&#39;s server components. This\n is great but we still need to perform side effects at some point, so how do\n we do that?\n\n The answer is through the `Effect` type that treats side effects as *data*.\n This approach is known as having **managed effects**: you pass data that\n describes a side effect to Lustre&#39;s runtime and it takes care of performing\n that effect and potentially sending messages back to your program for you.\n By going through this abstraction we discourage side effects from being\n performed in the middle of our program.\n\n ## Examples\n\n For folks coming from other languages (or other Gleam code!) where side\n effects are often performed in-place, this can feel a bit strange. A couple\n of the examples in the repo tackle effects:\n\n - [`05-http-requests`](https://github.com/lustre-labs/lustre/tree/main/examples/05-http-requests)\n - [`06-custom-effects`](https://github.com/lustre-labs/lustre/tree/main/examples/06-custom-effects)\n\n This list of examples is likely to grow over time, so be sure to check back\n every now and then to see what&#39;s new!\n\n ## Getting help\n\n If you&#39;re having trouble with Lustre or not sure what the right way to do\n something is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\n You could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n\n While our docs are still a work in progress, the official [Elm guide](https://guide.elm-lang.org)\n is also a great resource for learning about the Model-View-Update architecture\n and the kinds of patterns that Lustre is built around.\n","url":"lustre/effect.html"},{"doc":"lustre/attribute","title":"Attribute","content":"pub type Attribute(msg) =\n  vdom.Attribute(msg)\n The `Attribute` type encompasses HTML attributes, DOM properties, and\n event listeners.\n\n\nSynonyms:\nlustre/attribute.Attribute\nlustre/attribute Attribute","url":"lustre/attribute.html#Attribute"},{"doc":"lustre/attribute","title":"accept","content":"pub fn accept(types: List(String)) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.accept\nlustre/attribute accept","url":"lustre/attribute.html#accept"},{"doc":"lustre/attribute","title":"accept_charset","content":"pub fn accept_charset(types: List(String)) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.accept_charset\nlustre/attribute accept_charset","url":"lustre/attribute.html#accept_charset"},{"doc":"lustre/attribute","title":"action","content":"pub fn action(url: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.action\nlustre/attribute action","url":"lustre/attribute.html#action"},{"doc":"lustre/attribute","title":"alt","content":"pub fn alt(text: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.alt\nlustre/attribute alt","url":"lustre/attribute.html#alt"},{"doc":"lustre/attribute","title":"attribute","content":"pub fn attribute(name: String, value: String) -&gt; Attribute(a)\n Create an HTML attribute. This is like saying `element.setAttribute(&quot;class&quot;, &quot;wibble&quot;)`\n in JavaScript. Attributes will be rendered when calling [`element.to_string`](./element.html#to_string).\n\n **Note**: there is a subtle difference between attributes and properties. You\n can read more about the implications of this\n [here](https://github.com/lustre-labs/lustre/blob/main/pages/hints/attributes-vs-properties.md).\n\nSynonyms:\nlustre/attribute.attribute\nlustre/attribute attribute","url":"lustre/attribute.html#attribute"},{"doc":"lustre/attribute","title":"autocomplete","content":"pub fn autocomplete(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autocomplete\nlustre/attribute autocomplete","url":"lustre/attribute.html#autocomplete"},{"doc":"lustre/attribute","title":"autofocus","content":"pub fn autofocus(should_autofocus: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autofocus\nlustre/attribute autofocus","url":"lustre/attribute.html#autofocus"},{"doc":"lustre/attribute","title":"autoplay","content":"pub fn autoplay(should_autoplay: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.autoplay\nlustre/attribute autoplay","url":"lustre/attribute.html#autoplay"},{"doc":"lustre/attribute","title":"charset","content":"pub fn charset(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.charset\nlustre/attribute charset","url":"lustre/attribute.html#charset"},{"doc":"lustre/attribute","title":"checked","content":"pub fn checked(is_checked: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.checked\nlustre/attribute checked","url":"lustre/attribute.html#checked"},{"doc":"lustre/attribute","title":"class","content":"pub fn class(name: String) -&gt; Attribute(a)\n\n\n &gt; **Note**: unlike most attributes, multiple `class` attributes are merged\n &gt; with any existing other classes on an element.\n\nSynonyms:\nlustre/attribute.class\nlustre/attribute class","url":"lustre/attribute.html#class"},{"doc":"lustre/attribute","title":"classes","content":"pub fn classes(names: List(#(String, Bool))) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.classes\nlustre/attribute classes","url":"lustre/attribute.html#classes"},{"doc":"lustre/attribute","title":"cols","content":"pub fn cols(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.cols\nlustre/attribute cols","url":"lustre/attribute.html#cols"},{"doc":"lustre/attribute","title":"content","content":"pub fn content(text: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.content\nlustre/attribute content","url":"lustre/attribute.html#content"},{"doc":"lustre/attribute","title":"controls","content":"pub fn controls(visible: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.controls\nlustre/attribute controls","url":"lustre/attribute.html#controls"},{"doc":"lustre/attribute","title":"disabled","content":"pub fn disabled(is_disabled: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.disabled\nlustre/attribute disabled","url":"lustre/attribute.html#disabled"},{"doc":"lustre/attribute","title":"download","content":"pub fn download(filename: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.download\nlustre/attribute download","url":"lustre/attribute.html#download"},{"doc":"lustre/attribute","title":"enctype","content":"pub fn enctype(value: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.enctype\nlustre/attribute enctype","url":"lustre/attribute.html#enctype"},{"doc":"lustre/attribute","title":"for","content":"pub fn for(id: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.for\nlustre/attribute for","url":"lustre/attribute.html#for"},{"doc":"lustre/attribute","title":"form_action","content":"pub fn form_action(action: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.form_action\nlustre/attribute form_action","url":"lustre/attribute.html#form_action"},{"doc":"lustre/attribute","title":"form_enctype","content":"pub fn form_enctype(value: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.form_enctype\nlustre/attribute form_enctype","url":"lustre/attribute.html#form_enctype"},{"doc":"lustre/attribute","title":"form_method","content":"pub fn form_method(method: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.form_method\nlustre/attribute form_method","url":"lustre/attribute.html#form_method"},{"doc":"lustre/attribute","title":"form_novalidate","content":"pub fn form_novalidate(value: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.form_novalidate\nlustre/attribute form_novalidate","url":"lustre/attribute.html#form_novalidate"},{"doc":"lustre/attribute","title":"form_target","content":"pub fn form_target(target: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.form_target\nlustre/attribute form_target","url":"lustre/attribute.html#form_target"},{"doc":"lustre/attribute","title":"height","content":"pub fn height(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.height\nlustre/attribute height","url":"lustre/attribute.html#height"},{"doc":"lustre/attribute","title":"href","content":"pub fn href(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.href\nlustre/attribute href","url":"lustre/attribute.html#href"},{"doc":"lustre/attribute","title":"http_equiv","content":"pub fn http_equiv(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.http_equiv\nlustre/attribute http_equiv","url":"lustre/attribute.html#http_equiv"},{"doc":"lustre/attribute","title":"id","content":"pub fn id(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.id\nlustre/attribute id","url":"lustre/attribute.html#id"},{"doc":"lustre/attribute","title":"loop","content":"pub fn loop(should_loop: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.loop\nlustre/attribute loop","url":"lustre/attribute.html#loop"},{"doc":"lustre/attribute","title":"map","content":"pub fn map(attr: Attribute(a), f: fn(a) -&gt; b) -&gt; Attribute(b)\n The `Attribute` type is parameterised by the type of messages it can produce\n from events handlers. Sometimes you might end up with an attribute from a\n library or module that produces a different type of message: this function lets\n you map the messages produced from one type to another.\n\nSynonyms:\nlustre/attribute.map\nlustre/attribute map","url":"lustre/attribute.html#map"},{"doc":"lustre/attribute","title":"max","content":"pub fn max(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.max\nlustre/attribute max","url":"lustre/attribute.html#max"},{"doc":"lustre/attribute","title":"method","content":"pub fn method(method: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.method\nlustre/attribute method","url":"lustre/attribute.html#method"},{"doc":"lustre/attribute","title":"min","content":"pub fn min(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.min\nlustre/attribute min","url":"lustre/attribute.html#min"},{"doc":"lustre/attribute","title":"msg","content":"pub fn msg(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.msg\nlustre/attribute msg","url":"lustre/attribute.html#msg"},{"doc":"lustre/attribute","title":"name","content":"pub fn name(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.name\nlustre/attribute name","url":"lustre/attribute.html#name"},{"doc":"lustre/attribute","title":"none","content":"pub fn none() -&gt; Attribute(a)\n Create an empty attribute. This is not added to the DOM and not rendered when\n calling [`element.to_string`](./element.html#to_string), but it is useful for\n _conditionally_ adding attributes to an element.\n\nSynonyms:\nlustre/attribute.none\nlustre/attribute none","url":"lustre/attribute.html#none"},{"doc":"lustre/attribute","title":"novalidate","content":"pub fn novalidate(value: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.novalidate\nlustre/attribute novalidate","url":"lustre/attribute.html#novalidate"},{"doc":"lustre/attribute","title":"on","content":"pub fn on(\n  name: String,\n  handler: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.on\nlustre/attribute on","url":"lustre/attribute.html#on"},{"doc":"lustre/attribute","title":"open","content":"pub fn open(is_open: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.open\nlustre/attribute open","url":"lustre/attribute.html#open"},{"doc":"lustre/attribute","title":"pattern","content":"pub fn pattern(regex: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.pattern\nlustre/attribute pattern","url":"lustre/attribute.html#pattern"},{"doc":"lustre/attribute","title":"placeholder","content":"pub fn placeholder(text: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.placeholder\nlustre/attribute placeholder","url":"lustre/attribute.html#placeholder"},{"doc":"lustre/attribute","title":"property","content":"pub fn property(name: String, value: a) -&gt; Attribute(b)\n Create a DOM property. This is like saying `element.className = &quot;wibble&quot;` in\n JavaScript. Properties will be **not** be rendered when calling\n [`element.to_string`](./element.html#to_string).\n\n **Note**: there is a subtle difference between attributes and properties. You\n can read more about the implications of this\n [here](https://github.com/lustre-labs/lustre/blob/main/pages/hints/attributes-vs-properties.md).\n\nSynonyms:\nlustre/attribute.property\nlustre/attribute property","url":"lustre/attribute.html#property"},{"doc":"lustre/attribute","title":"readonly","content":"pub fn readonly(is_readonly: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.readonly\nlustre/attribute readonly","url":"lustre/attribute.html#readonly"},{"doc":"lustre/attribute","title":"rel","content":"pub fn rel(relationship: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.rel\nlustre/attribute rel","url":"lustre/attribute.html#rel"},{"doc":"lustre/attribute","title":"required","content":"pub fn required(is_required: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.required\nlustre/attribute required","url":"lustre/attribute.html#required"},{"doc":"lustre/attribute","title":"role","content":"pub fn role(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.role\nlustre/attribute role","url":"lustre/attribute.html#role"},{"doc":"lustre/attribute","title":"rows","content":"pub fn rows(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.rows\nlustre/attribute rows","url":"lustre/attribute.html#rows"},{"doc":"lustre/attribute","title":"selected","content":"pub fn selected(is_selected: Bool) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.selected\nlustre/attribute selected","url":"lustre/attribute.html#selected"},{"doc":"lustre/attribute","title":"src","content":"pub fn src(uri: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.src\nlustre/attribute src","url":"lustre/attribute.html#src"},{"doc":"lustre/attribute","title":"step","content":"pub fn step(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.step\nlustre/attribute step","url":"lustre/attribute.html#step"},{"doc":"lustre/attribute","title":"style","content":"pub fn style(properties: List(#(String, String))) -&gt; Attribute(a)\n\n\n &gt; **Note**: unlike most attributes, multiple `style` attributes are merged\n &gt; with any existing other styles on an element. Styles added _later_ in the\n &gt; list will override styles added earlier.\n\nSynonyms:\nlustre/attribute.style\nlustre/attribute style","url":"lustre/attribute.html#style"},{"doc":"lustre/attribute","title":"target","content":"pub fn target(target: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.target\nlustre/attribute target","url":"lustre/attribute.html#target"},{"doc":"lustre/attribute","title":"title","content":"pub fn title(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.title\nlustre/attribute title","url":"lustre/attribute.html#title"},{"doc":"lustre/attribute","title":"type_","content":"pub fn type_(name: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.type_\nlustre/attribute type_","url":"lustre/attribute.html#type_"},{"doc":"lustre/attribute","title":"value","content":"pub fn value(val: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.value\nlustre/attribute value","url":"lustre/attribute.html#value"},{"doc":"lustre/attribute","title":"width","content":"pub fn width(val: Int) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.width\nlustre/attribute width","url":"lustre/attribute.html#width"},{"doc":"lustre/attribute","title":"wrap","content":"pub fn wrap(mode: String) -&gt; Attribute(a)\n\nSynonyms:\nlustre/attribute.wrap\nlustre/attribute wrap","url":"lustre/attribute.html#wrap"},{"doc":"lustre/attribute","title":"lustre/attribute","content":"","url":"lustre/attribute.html"},{"doc":"lustre/element","title":"Element","content":"pub type Element(msg) =\n  vdom.Element(msg)\n The `Element` type is how Lustre represents chunks of HTML. The `msg` type\n variable is used to represent the types of messages that can be produced from\n events on the element or its children.\n\n **Note**: Just because an element _can_ produces messages of a given type,\n doesn&#39;t mean that it _will_! The `msg` type variable is used to represent the\n potential for messages to be produced, not a guarantee.\n\n The most basic ways to create elements are:\n\n - The [`element`](#element) function to construct arbitrary HTML elements.\n   You can also use this render Custom Elements (like those registered as\n   Lustre components).\n\n - The [`text`](#text) function to turn a Gleam string into a text node.\n\n - The [`none`](#none) function to render nothing - useful for conditional\n   rendering.\n\n If you have more complex needs, there are two more-advanced functions:\n\n - The [`namespaced`](#namespaced) function to create elements in a specific\n   XML namespace. This is useful for SVG or MathML elements, for example.\n\n - The [`advanced`](#advanced) function to create elements with more control\n   over how the element is rendered when converted to a string. This is\n   necessary because some HTML, SVG, and MathML elements are self-closing or\n   void elements, and Lustre needs to know how to render them correctly!\n\n For most applications, you&#39;ll only need to use the simpler functions; usually\n the [`text`](#text) and [`none`](#none) functions are enough. This is because\n Lustre already provides a module with all the standard HTML and SVG elements\n ready to use in [`lustre/element/html`](./element/html.html) and\n [`lustre/element/svg`](./element/svg.html).\n\n\nSynonyms:\nlustre/element.Element\nlustre/element Element","url":"lustre/element.html#Element"},{"doc":"lustre/element","title":"advanced","content":"pub fn advanced(\n  namespace: String,\n  tag: String,\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n  self_closing: Bool,\n  void: Bool,\n) -&gt; Element(a)\n A function for constructing elements with more control over how the element\n is rendered when converted to a string. This is necessary because some HTML,\n SVG, and MathML elements are self-closing or void elements, and Lustre needs\n to know how to render them correctly!\n\nSynonyms:\nlustre/element.advanced\nlustre/element advanced","url":"lustre/element.html#advanced"},{"doc":"lustre/element","title":"element","content":"pub fn element(\n  tag: String,\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n A general function for constructing any kind of element. In most cases you\n will want to use the [`lustre/element/html`](./element/html.html) instead but this\n function is particularly handy when constructing custom elements, either\n from your own Lustre components or from external JavaScript libraries.\n\n **Note**: Because Lustre is primarily used to create HTML, this function\n special-cases the following tags which render as\n [void elements](https://developer.mozilla.org/en-US/docs/Glossary/Void_element):\n\n   - area\n   - base\n   - br\n   - col\n   - embed\n   - hr\n   - img\n   - input\n   - link\n   - meta\n   - param\n   - source\n   - track\n   - wbr\n\n  This will only affect the output of `to_string` and `to_string_builder`!\n  If you need to render any of these tags with children, *or* you want to\n  render some other tag as self-closing or void, use [`advanced`](#advanced)\n  to construct the element instead.\n\nSynonyms:\nlustre/element.element\nlustre/element element","url":"lustre/element.html#element"},{"doc":"lustre/element","title":"fragment","content":"pub fn fragment(elements: List(Element(a))) -&gt; Element(a)\n A function for wrapping elements to be rendered within a parent container without\n specififying the container on definition. Allows the treatment of List(Element(msg))\n as if it were Element(msg). Useful when generating a list of elements from data but\n used downstream.\n\nSynonyms:\nlustre/element.fragment\nlustre/element fragment","url":"lustre/element.html#fragment"},{"doc":"lustre/element","title":"keyed","content":"pub fn keyed(\n  el: fn(List(Element(a))) -&gt; Element(a),\n  children: List(#(String, Element(a))),\n) -&gt; Element(a)\n Keying elements is an optimisation that helps the runtime reuse existing DOM\n nodes in cases where children are reordered or removed from a list. Maybe you\n have a list of elements that can be filtered or sorted in some way, or additions\n to the front are common. In these cases, keying elements can help Lustre avoid\n unecessary DOM manipulations by pairing the DOM nodes with the elements in the\n list that share the same key.\n\n You can easily take an element from `lustre/element/html` and key its children\n by making use of Gleam&#39;s [function capturing syntax](https://tour.gleam.run/functions/function-captures/):\n\n \n import gleam/list\n import lustre/element\n import lustre/element/html\n\n fn example() {\n   element.keyed(html.ul([], _), {\n     use item &lt;- list.map(todo_list)\n     let child = html.li([], [view_item(item)])\n\n     #(item.id, child)\n   })\n }\n \n\n **Note**: The key must be unique within the list of children, but it doesn&#39;t\n have to be unique across the whole application. It&#39;s fine to use the same key\n in different lists. Lustre will display a warning in the browser console when\n it detects duplicate keys in a list.\n\n\nSynonyms:\nlustre/element.keyed\nlustre/element keyed","url":"lustre/element.html#keyed"},{"doc":"lustre/element","title":"map","content":"pub fn map(element: Element(a), f: fn(a) -&gt; b) -&gt; Element(b)\n The `Element` type is parameterised by the type of messages it can produce\n from events. Sometimes you might end up with a fragment of HTML from another\n library or module that produces a different type of message: this function lets\n you map the messages produced from one type to another.\n\n Think of it like `list.map` or `result.map` but for HTML events!\n\nSynonyms:\nlustre/element.map\nlustre/element map","url":"lustre/element.html#map"},{"doc":"lustre/element","title":"namespaced","content":"pub fn namespaced(\n  namespace: String,\n  tag: String,\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n A function for constructing elements in a specific XML namespace. This can\n be used to construct SVG or MathML elements, for example.\n\nSynonyms:\nlustre/element.namespaced\nlustre/element namespaced","url":"lustre/element.html#namespaced"},{"doc":"lustre/element","title":"none","content":"pub fn none() -&gt; Element(a)\n A function for rendering nothing. This is mostly useful for conditional\n rendering, where you might want to render something only if a certain\n condition is met.\n\nSynonyms:\nlustre/element.none\nlustre/element none","url":"lustre/element.html#none"},{"doc":"lustre/element","title":"text","content":"pub fn text(content: String) -&gt; Element(a)\n A function for turning a Gleam string into a text node. Gleam doesn&#39;t have\n union types like some other languages you may be familiar with, like TypeScript.\n Instead, we need a way to take a `String` and turn it into an `Element` somehow:\n this function is exactly that!\n\nSynonyms:\nlustre/element.text\nlustre/element text","url":"lustre/element.html#text"},{"doc":"lustre/element","title":"to_document_string","content":"pub fn to_document_string(el: Element(a)) -&gt; String\n Converts an element to a string like [`to_string`](#to_string), but prepends\n a `&lt;!doctype html&gt;` declaration to the string. This is useful for rendering\n complete HTML documents.\n\n If the provided element is not an `html` element, it will be wrapped in both\n a `html` and `body` element.\n\nSynonyms:\nlustre/element.to_document_string\nlustre/element to_document_string","url":"lustre/element.html#to_document_string"},{"doc":"lustre/element","title":"to_document_string_builder","content":"pub fn to_document_string_builder(el: Element(a)) -&gt; StringTree\n Converts an element to a `StringTree` like [`to_string_builder`](#to_string_builder),\n but prepends a `&lt;!doctype html&gt;` declaration. This is useful for rendering\n complete HTML documents.\n\n If the provided element is not an `html` element, it will be wrapped in both\n a `html` and `body` element.\n\nSynonyms:\nlustre/element.to_document_string_builder\nlustre/element to_document_string_builder","url":"lustre/element.html#to_document_string_builder"},{"doc":"lustre/element","title":"to_readable_string","content":"pub fn to_readable_string(el: Element(a)) -&gt; String\n Converts a Lustre `Element` to a human-readable string by inserting new lines\n and indentation where appropriate. This is useful for debugging and testing,\n but for production code you should use [`to_string`](#to_string) or\n [`to_document_string`](#to_document_string) instead.\n\n 💡 This function works great with the snapshot testing library\n    [birdie](https://hexdocs.pm/birdie)!\n\n ## Using `to_string`:\n\n html\n &lt;header&gt;&lt;h1&gt;Hello, world!&lt;/h1&gt;&lt;/header&gt;\n \n\n ## Using `to_readable_string`\n\n html\n &lt;header&gt;\n   &lt;h1&gt;\n     Hello, world!\n   &lt;/h1&gt;\n &lt;/header&gt;\n \n\nSynonyms:\nlustre/element.to_readable_string\nlustre/element to_readable_string","url":"lustre/element.html#to_readable_string"},{"doc":"lustre/element","title":"to_string","content":"pub fn to_string(element: Element(a)) -&gt; String\n Convert a Lustre `Element` to a string. This is _not_ pretty-printed, so\n there are no newlines or indentation. If you need to pretty-print an element,\n reach out on the [Gleam Discord](https://discord.gg/Fm8Pwmy) or\n [open an issue](https://github.com/lustre-labs/lustre/issues/new) with your\n use case and we&#39;ll see what we can do!\n\nSynonyms:\nlustre/element.to_string\nlustre/element to_string","url":"lustre/element.html#to_string"},{"doc":"lustre/element","title":"to_string_builder","content":"pub fn to_string_builder(element: Element(a)) -&gt; StringTree\n Convert a Lustre `Element` to a `StringTree`. This is _not_ pretty-printed,\n so there are no newlines or indentation. If you need to pretty-print an element,\n reach out on the [Gleam Discord](https://discord.gg/Fm8Pwmy) or\n [open an issue](https://github.com/lustre-labs/lustre/issues/new) with your\n use case and we&#39;ll see what we can do!\n\nSynonyms:\nlustre/element.to_string_builder\nlustre/element to_string_builder","url":"lustre/element.html#to_string_builder"},{"doc":"lustre/element","title":"lustre/element","content":" Lustre wouldn&#39;t be much use as a frontend framework if it didn&#39;t provide a\n way to create HTML elements. This module contains the basic functions\n necessary to construct and manipulate different HTML elements.\n\n It is also possible to use Lustre as a HTML templating library, without\n using its runtime or framework features.\n","url":"lustre/element.html"},{"doc":"lustre","title":"Action","content":"pub type Action(msg, runtime) =\n  runtime.Action(msg, runtime)\n An action represents a message that can be sent to a running Lustre application.\n Code that is orchestrating an application where Lustre is only one part of the\n system will likely want to send actions to the Lustre runtime. For most kinds of\n application, you can usually ignore actions entirely.\n\n The `msg` type parameter is the kind of messages you can send to the runtime&#39;s\n `update` function through the [`dispatch`](#dispatch) action.\n\n The `runtime` type parameter represents the type of Lustre application that\n can receive this action. If we [`start`](#start) a typical Lustre SPA, we\n get back the type `Result(fn(Action(msg, ClientSpa)) -&gt; Nil, Error)`. This\n means we can only send actions suitable for the [`ClientSpa`](#ClientSpa)\n runtime, and trying to send actions like [`add_renderer`](#add_renderer) would\n result in a type error.\n\n\nSynonyms:\nlustre.Action\nlustre Action","url":"lustre.html#Action"},{"doc":"lustre","title":"App","content":"pub opaque type App(flags, model, msg)\n Represents a constructed Lustre application that is ready to be started.\n Depending on where you want the application to run, you have a few options:\n\n - Use [`start`](#start) to start a single-page-application in the browser.\n\n   This is the most common way to start a Lustre application. If you&#39;re new to\n   Lustre or frontend development in general, make sure you check out the\n   [examples](https://github.com/lustre-labs/lustre/tree/main/examples) or the\n   [quickstart guide]()\n\n - Use [`start_server_component`](#start_server_component) to start a Lustre\n   Server Component anywhere Gleam will run: Erlang, Node, Deno, or in the\n   browser. If you&#39;re running on the BEAM though, you should...\n\n - Use [`start_actor`](#start_actor) to start a Lustre Server Component only\n   for the Erlang target. BEAM users should always prefer this over\n   `start_server_component` so they can take advantage of OTP features.\n\n - Use [`register`](#register) to register a component in the browser to be\n   used as a Custom Element. This is useful even if you&#39;re not using Lustre\n   to build a SPA.\n\n If you&#39;re only interested in using Lustre as a HTML templating engine, you\n don&#39;t need an `App` at all! You can render an element directly using the\n [`element.to_string`](./lustre/element.html#to_string) function.\n\n\nSynonyms:\nlustre.App\nlustre App","url":"lustre.html#App"},{"doc":"lustre","title":"ClientSpa","content":"pub type ClientSpa\n The `ClientSpa` runtime is the most typical kind of Lustre application: it&#39;s\n a single-page application that runs in the browser similar to React or Elm.\n\n This type is used to tag the [`Action`](#Action) type to stop you accidentally\n sending actions to the wrong kind of runtime.\n\n\nSynonyms:\nlustre.ClientSpa\nlustre ClientSpa","url":"lustre.html#ClientSpa"},{"doc":"lustre","title":"Error","content":"pub type Error {\n  ActorError(StartError)\n  BadComponentName(name: String)\n  ComponentAlreadyRegistered(name: String)\n  ElementNotFound(selector: String)\n  NotABrowser\n  NotErlang\n}\n Starting a Lustre application might fail for a number of reasons. This error\n type enumerates all those reasons, even though some of them are only possible\n on certain targets.\n\nActorError(StartError)\n\n\nBadComponentName(name: String)\n\n\nComponentAlreadyRegistered(name: String)\n\n\nElementNotFound(selector: String)\n\n\nNotABrowser\n\n\nNotErlang\n\n\nSynonyms:\nlustre.Error\nlustre Error","url":"lustre.html#Error"},{"doc":"lustre","title":"Patch","content":"pub type Patch(msg) =\n  patch.Patch(msg)\n Patches are sent by server components to any connected renderers. Because\n server components are not opinionated about your network layer or how your\n wider application is organised, it is your responsibility to make sure a `Patch`\n makes its way to the server component client runtime.\n\n\nSynonyms:\nlustre.Patch\nlustre Patch","url":"lustre.html#Patch"},{"doc":"lustre","title":"ServerComponent","content":"pub type ServerComponent\n A `ServerComponent` is a type of Lustre application that does not directly\n render anything to the DOM. Instead, it can run anywhere Gleam runs and\n operates in a &quot;headless&quot; mode where it computes diffs between renders and\n sends them to any number of connected listeners.\n\n Lustre Server Components are not tied to any particular transport or network\n protocol, but they are most commonly used with WebSockets in a fashion similar\n to Phoenix LiveView.\n\n This type is used to tag the [`Action`](#Action) type to stop you accidentally\n sending actions to the wrong kind of runtime.\n\n\nSynonyms:\nlustre.ServerComponent\nlustre ServerComponent","url":"lustre.html#ServerComponent"},{"doc":"lustre","title":"application","content":"pub fn application(\n  init: fn(a) -&gt; #(b, Effect(c)),\n  update: fn(b, c) -&gt; #(b, Effect(c)),\n  view: fn(b) -&gt; Element(c),\n) -&gt; App(a, b, c)\n A complete Lustre application that follows the Model-View-Update architecture\n and can handle side effects like HTTP requests or querying the DOM. Most real\n Lustre applications will use this constructor.\n\n To learn more about effects and their purpose, take a look at the\n [`effect`](./lustre/effect.html) module or the\n [HTTP requests example](https://github.com/lustre-labs/lustre/tree/main/examples/05-http-requests).\n\nSynonyms:\nlustre.application\nlustre application","url":"lustre.html#application"},{"doc":"lustre","title":"component","content":"pub fn component(\n  init: fn(a) -&gt; #(b, Effect(c)),\n  update: fn(b, c) -&gt; #(b, Effect(c)),\n  view: fn(b) -&gt; Element(c),\n  on_attribute_change: Dict(\n    String,\n    fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n  ),\n) -&gt; App(a, b, c)\n A `component` is a type of Lustre application designed to be embedded within\n another application and has its own encapsulated update loop. This constructor\n is almost identical to the [`application`](#application) constructor, but it\n also allows you to specify a dictionary of attribute names and decoders.\n\n When a component is rendered in a parent application, it can receive data from\n the parent application through HTML attributes and properties just like any\n other HTML element. This dictionary of decoders allows you to specify how to\n decode those attributes into messages your component&#39;s update loop can handle.\n\n **Note**: Lustre components are conceptually a lot &quot;heavier&quot; than components\n in frameworks like React. They should be used for more complex UI widgets\n like a combobox with complex keyboard interactions rather than simple things\n like buttons or text inputs. Where possible try to think about how to build\n your UI with simple view functions (functions that return [Elements](./lustre/element.html#Element))\n and only reach for components when you really need to encapsulate that update\n loop.\n\nSynonyms:\nlustre.component\nlustre component","url":"lustre.html#component"},{"doc":"lustre","title":"dispatch","content":"pub fn dispatch(msg: a) -&gt; Action(a, b)\n Dispatch a message to a running application&#39;s `update` function. This can be\n used as a way for the outside world to communicate with a Lustre app without\n the app needing to initiate things with an effect.\n\n Both client SPAs and server components can have messages sent to them using\n the `dispatch` action.\n\nSynonyms:\nlustre.dispatch\nlustre dispatch","url":"lustre.html#dispatch"},{"doc":"lustre","title":"element","content":"pub fn element(html: Element(a)) -&gt; App(Nil, Nil, a)\n An element is the simplest type of Lustre application. It renders its contents\n once and does not handle any messages or effects. Often this type of application\n is used for folks just getting started with Lustre on the frontend and want a\n quick way to get something on the screen.\n\n Take a look at the [`simple`](#simple) application constructor if you want to\n build something interactive.\n\n &gt; **Note**: Just because an element doesn&#39;t have its own update loop, doesn&#39;t\n &gt; mean its content is always static! An element application may render a client\n &gt; or server component that has its own encapsulated update loop!\n\nSynonyms:\nlustre.element\nlustre element","url":"lustre.html#element"},{"doc":"lustre","title":"is_browser","content":"pub fn is_browser() -&gt; Bool\n Gleam&#39;s conditional compilation makes it possible to have different implementations\n of a function for different targets, but it&#39;s not possible to know what runtime\n you&#39;re targeting at compile-time.\n\n This is problematic if you&#39;re using server components with a JavaScript\n backend because you&#39;ll want to know whether you&#39;re currently running on your\n server or in the browser: this function tells you that!\n\nSynonyms:\nlustre.is_browser\nlustre is_browser","url":"lustre.html#is_browser"},{"doc":"lustre","title":"is_registered","content":"pub fn is_registered(name: String) -&gt; Bool\n Check if the given component name has already been registered as a Custom\n Element. This is particularly useful in contexts where _other web components_\n may have been registered and you must avoid collisions.\n\nSynonyms:\nlustre.is_registered\nlustre is_registered","url":"lustre.html#is_registered"},{"doc":"lustre","title":"register","content":"pub fn register(\n  app: App(Nil, a, b),\n  name: String,\n) -&gt; Result(Nil, Error)\n Register a Lustre application as a Web Component. This lets you render that\n application in another Lustre application&#39;s view or use it as a Custom Element\n outside of Lustre entirely.The provided application can only have `Nil` flags\n because there is no way to provide an initial value for flags when using a\n Custom Element!\n\n The second argument is the name of the Custom Element. This is the name you&#39;d\n use in HTML to render the component. For example, if you register a component\n with the name `my-component`, you&#39;d use it in HTML by writing `&lt;my-component&gt;`\n or in Lustre by rendering `element(&quot;my-component&quot;, [], [])`.\n\n **Note**: There are [some rules](https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry/define#valid_custom_element_names)\n for what names are valid for a Custom Element. The most important one is that\n the name *must* contain a hypen so that it can be distinguished from standard\n HTML elements.\n\n **Note**: This function is only meaningful when running in the browser and will\n produce a `NotABrowser` error if called anywhere else. For server contexts,\n you can render a Lustre server component using [`start_server_component`](#start_server_component)\n or [`start_actor`](#start_actor) instead.\n\nSynonyms:\nlustre.register\nlustre register","url":"lustre.html#register"},{"doc":"lustre","title":"shutdown","content":"pub fn shutdown() -&gt; Action(a, b)\n Instruct a running application to shut down. For client SPAs this will stop\n the runtime and unmount the app from the DOM. For server components, this will\n stop the runtime and prevent any further patches from being sent to connected\n clients.\n\nSynonyms:\nlustre.shutdown\nlustre shutdown","url":"lustre.html#shutdown"},{"doc":"lustre","title":"simple","content":"pub fn simple(\n  init: fn(a) -&gt; b,\n  update: fn(b, c) -&gt; b,\n  view: fn(b) -&gt; Element(c),\n) -&gt; App(a, b, c)\n A `simple` application has the basic Model-View-Update building blocks present\n in all Lustre applications, but it cannot handle effects. This is a great way\n to learn the basics of Lustre and its architecture.\n\n Once you&#39;re comfortable with the Model-View-Update loop and want to start\n building more complex applications that can communicate with the outside world,\n you&#39;ll want to use the [`application`](#application) constructor instead.\n\nSynonyms:\nlustre.simple\nlustre simple","url":"lustre.html#simple"},{"doc":"lustre","title":"start","content":"pub fn start(\n  app: App(a, b, c),\n  onto selector: String,\n  with flags: a,\n) -&gt; Result(fn(Action(c, ClientSpa)) -&gt; Nil, Error)\n Start a constructed application as a client-side single-page application (SPA).\n This is the most typical way to start a Lustre application and will *only* work\n in the browser\n\n The second argument is a [CSS selector](https://developer.mozilla.org/en-US/docs/Web/API/Document/querySelector)\n used to locate the DOM element where the application will be mounted on to.\n The most common selectors are `&quot;#app&quot;` to target an element with an id of `app`\n or `[data-lustre-app]` to target an element with a `data-lustre-app` attribute.\n\n The third argument is the starting data for the application. This is passed\n to the application&#39;s `init` function.\n\nSynonyms:\nlustre.start\nlustre start","url":"lustre.html#start"},{"doc":"lustre","title":"start_actor","content":"pub fn start_actor(\n  app: App(a, b, c),\n  with flags: a,\n) -&gt; Result(Subject(Action(c, ServerComponent)), Error)\n Start an application as a server component specifically for the Erlang target.\n Instead of receiving a callback on successful start, this function returns\n a [`Subject`](https://hexdocs.pm/gleam_erlang/gleam/erlang/process.html#Subject)\n\n\n **Note**: This function is only meaningful on the Erlang target. Attempts to\n call it on the JavaScript will result in the `NotErlang` error. If you&#39;re running\n a Lustre server component on Node or Deno, use [`start_server_component`](#start_server_component)\n instead.\n\nSynonyms:\nlustre.start_actor\nlustre start_actor","url":"lustre.html#start_actor"},{"doc":"lustre","title":"start_server_component","content":"pub fn start_server_component(\n  app: App(a, b, c),\n  with flags: a,\n) -&gt; Result(fn(Action(c, ServerComponent)) -&gt; Nil, Error)\n Start an application as a server component. This runs in a headless mode and\n doesn&#39;t render anything to the DOM. Instead, multiple clients can be attached\n using the [`add_renderer`](#add_renderer) action.\n\n If a server component starts successfully, this function will return a callback\n that can be used to send actions to the component runtime.\n\n A server component will keep running until the program is terminated or the\n [`shutdown`](#shutdown) action is sent to it.\n\n **Note**: Users running their application on the BEAM should use [`start_actor`](#start_actor)\n instead to make use of Gleam&#39;s OTP abstractions.\n\nSynonyms:\nlustre.start_server_component\nlustre start_server_component","url":"lustre.html#start_server_component"},{"doc":"lustre","title":"lustre","content":" Lustre is a framework for rendering Web applications and components using\n Gleam. This module contains the core API for constructing and communicating\n with Lustre applications. If you&#39;re new to Lustre or frontend development in\n general, make sure you check out the [examples](https://github.com/lustre-labs/lustre/tree/main/examples)\n or the [quickstart guide](./guide/01-quickstart.html) to get up to speed!\n\n Lustre currently has three kinds of application:\n\n 1. A client-side single-page application: think Elm or React or Vue. These\n    are applications that run in the client&#39;s browser and are responsible for\n    rendering the entire page.\n\n 2. A client-side component: an encapsulated Lustre application that can be\n    rendered inside another Lustre application as a Web Component. Communication\n    happens via attributes and event listeners, like any other encapsulated\n    HTML element.\n\n 3. A server component. These are applications that run anywhere Gleam runs\n    and communicate with any number of connected clients by sending them\n    patches to apply to their DOM.\n\n    There are two pieces to a server component: the main server component\n    runtime that contains your application logic, and a client-side runtime\n    that listens for patches over a WebSocket and applies them to the DOM.\n\n    The server component runtime can run anywhere Gleam does, but the\n    client-side runtime must be run in a browser. To use it either render the\n    [provided script element](./lustre/server_component.html#script) or use the script files\n    from Lustre&#39;s `priv/` directory directly.\n\n No matter where a Lustre application runs, it will always follow the same\n Model-View-Update architecture. Popularised by Elm (where it is known as The\n Elm Architecture), this pattern has since made its way into many other\n languages and frameworks and has proven to be a robust and reliable way to\n build complex user interfaces.\n\n There are three main building blocks to the Model-View-Update architecture:\n\n - A `Model` that represents your application&#39;s state and an `init` function\n   to create it.\n\n - A `Msg` type that represents all the different ways the outside world can\n   communicate with your application and an `update` function that modifies\n   your model in response to those messages.\n\n - A `view` function that renders your model to HTML, represented as an\n   `Element`.\n\n To see how those pieces fit together, here&#39;s a little diagram:\n\n ```text\n                                          +--------+\n                                          |        |\n                                          | update |\n                                          |        |\n                                          +--------+\n                                            ^    |\n                                            |    |\n                                        Msg |    | #(Model, Effect(Msg))\n                                            |    |\n                                            |    v\n +------+                         +------------------------+\n |      |  #(Model, Effect(Msg))  |                        |\n | init |------------------------&gt;|     Lustre Runtime     |\n |      |                         |                        |\n +------+                         +------------------------+\n                                            ^    |\n                                            |    |\n                                        Msg |    | Model\n                                            |    |\n                                            |    v\n                                          +--------+\n                                          |        |\n                                          |  view  |\n                                          |        |\n                                          +--------+\n ```\n\n The `Effect` type here encompasses things like HTTP requests and other kinds\n of communication with the &quot;outside world&quot;. You can read more about effects\n and their purpose in the [`effect`](./effect.html) module.\n\n For many kinds of apps, you can take these three building blocks and put\n together a Lustre application capable of running *anywhere*. Because of that,\n we like to describe Lustre as a **universal framework**.\n\n ## Guides\n\n A number of guides have been written to teach you how to use Lustre to build\n different kinds of applications. If you&#39;re just getting started with Lustre\n or frontend development, we recommend reading through them in order:\n\n - [`01-quickstart`](/guide/01-quickstart.html)\n\n This list of guides is likely to grow over time, so be sure to check back\n every now and then to see what&#39;s new!\n\n ## Examples\n\n If you prefer to learn by seeing and adapting existing code, there are also\n a number of examples in the [Lustre GitHub repository](https://github.com/lustre-labs/lustre)\n that each demonstrate a different concept or idea:\n\n - [`01-hello-world`](https://github.com/lustre-labs/lustre/tree/main/examples/01-hello-world)\n - [`02-interactivity`](https://github.com/lustre-labs/lustre/tree/main/examples/02-interactivity)\n - [`03-controlled-inputs`](https://github.com/lustre-labs/lustre/tree/main/examples/03-controlled-inputs)\n - [`04-custom-event-handlers`](https://github.com/lustre-labs/lustre/tree/main/examples/04-custom-event-handlers)\n - [`05-http-requests`](https://github.com/lustre-labs/lustre/tree/main/examples/05-http-requests)\n - [`06-custom-effects`](https://github.com/lustre-labs/lustre/tree/main/examples/06-custom-effects)\n\n This list of examples is likely to grow over time, so be sure to check back\n every now and then to see what&#39;s new!\n\n ## Companion libraries\n\n While this package contains the runtime and API necessary for building and\n rendering applications, there is also a small collection of companion libraries\n built to make building Lustre applications easier:\n\n - [lustre/ui](https://github.com/lustre-labs/ui) is a collection of pre-designed\n   elements and design tokens for building user interfaces with Lustre.\n\n - [lustre/ssg](https://github.com/lustre-labs/ssg) is a simple static site\n   generator that you can use to produce static HTML documents from your Lustre\n   applications.\n\n Both of these packages are heavy works in progress: any feedback or contributions\n are very welcome!\n\n\n ## Getting help\n\n If you&#39;re having trouble with Lustre or not sure what the right way to do\n something is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\n You could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n\n While our docs are still a work in progress, the official [Elm guide](https://guide.elm-lang.org)\n is also a great resource for learning about the Model-View-Update architecture\n and the kinds of patterns that Lustre is built around.\n\n ## Contributing\n\n The best way to contribute to Lustre is by building things! If you&#39;ve built\n something cool with Lustre you want to share then please share it on the\n `#sharing` channel in the  [Gleam Discord server](https://discord.gg/Fm8Pwmy).\n You can also tag Hayleigh on Twitter [@hayleigh-dot-dev](https://twitter.com/hayleighdotdev)\n or on BlueSky [@hayleigh.dev](https://bsky.app/profile/hayleigh.dev).\n\n If you run into any issues or have ideas for how to improve Lustre, please\n open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n Fixes and improvements to the documentation are also very welcome!\n\n Finally, if you&#39;d like, you can support the project through\n [GitHub Sponsors](https://github.com/sponsors/hayleigh-dot-dev). Sponsorship\n helps fund the copious amounts of coffee that goes into building and maintaining\n Lustre, and is very much appreciated!\n","url":"lustre.html"},{"doc":"lustre/element/html","title":"a","content":"pub fn a(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.a\nlustre/element/html a","url":"lustre/element/html.html#a"},{"doc":"lustre/element/html","title":"abbr","content":"pub fn abbr(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.abbr\nlustre/element/html abbr","url":"lustre/element/html.html#abbr"},{"doc":"lustre/element/html","title":"address","content":"pub fn address(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.address\nlustre/element/html address","url":"lustre/element/html.html#address"},{"doc":"lustre/element/html","title":"area","content":"pub fn area(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.area\nlustre/element/html area","url":"lustre/element/html.html#area"},{"doc":"lustre/element/html","title":"article","content":"pub fn article(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.article\nlustre/element/html article","url":"lustre/element/html.html#article"},{"doc":"lustre/element/html","title":"aside","content":"pub fn aside(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.aside\nlustre/element/html aside","url":"lustre/element/html.html#aside"},{"doc":"lustre/element/html","title":"audio","content":"pub fn audio(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.audio\nlustre/element/html audio","url":"lustre/element/html.html#audio"},{"doc":"lustre/element/html","title":"b","content":"pub fn b(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.b\nlustre/element/html b","url":"lustre/element/html.html#b"},{"doc":"lustre/element/html","title":"base","content":"pub fn base(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.base\nlustre/element/html base","url":"lustre/element/html.html#base"},{"doc":"lustre/element/html","title":"bdi","content":"pub fn bdi(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.bdi\nlustre/element/html bdi","url":"lustre/element/html.html#bdi"},{"doc":"lustre/element/html","title":"bdo","content":"pub fn bdo(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.bdo\nlustre/element/html bdo","url":"lustre/element/html.html#bdo"},{"doc":"lustre/element/html","title":"blockquote","content":"pub fn blockquote(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.blockquote\nlustre/element/html blockquote","url":"lustre/element/html.html#blockquote"},{"doc":"lustre/element/html","title":"body","content":"pub fn body(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.body\nlustre/element/html body","url":"lustre/element/html.html#body"},{"doc":"lustre/element/html","title":"br","content":"pub fn br(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.br\nlustre/element/html br","url":"lustre/element/html.html#br"},{"doc":"lustre/element/html","title":"button","content":"pub fn button(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.button\nlustre/element/html button","url":"lustre/element/html.html#button"},{"doc":"lustre/element/html","title":"canvas","content":"pub fn canvas(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.canvas\nlustre/element/html canvas","url":"lustre/element/html.html#canvas"},{"doc":"lustre/element/html","title":"caption","content":"pub fn caption(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.caption\nlustre/element/html caption","url":"lustre/element/html.html#caption"},{"doc":"lustre/element/html","title":"cite","content":"pub fn cite(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.cite\nlustre/element/html cite","url":"lustre/element/html.html#cite"},{"doc":"lustre/element/html","title":"code","content":"pub fn code(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.code\nlustre/element/html code","url":"lustre/element/html.html#code"},{"doc":"lustre/element/html","title":"col","content":"pub fn col(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.col\nlustre/element/html col","url":"lustre/element/html.html#col"},{"doc":"lustre/element/html","title":"colgroup","content":"pub fn colgroup(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.colgroup\nlustre/element/html colgroup","url":"lustre/element/html.html#colgroup"},{"doc":"lustre/element/html","title":"data","content":"pub fn data(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.data\nlustre/element/html data","url":"lustre/element/html.html#data"},{"doc":"lustre/element/html","title":"datalist","content":"pub fn datalist(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.datalist\nlustre/element/html datalist","url":"lustre/element/html.html#datalist"},{"doc":"lustre/element/html","title":"dd","content":"pub fn dd(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.dd\nlustre/element/html dd","url":"lustre/element/html.html#dd"},{"doc":"lustre/element/html","title":"del","content":"pub fn del(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.del\nlustre/element/html del","url":"lustre/element/html.html#del"},{"doc":"lustre/element/html","title":"details","content":"pub fn details(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.details\nlustre/element/html details","url":"lustre/element/html.html#details"},{"doc":"lustre/element/html","title":"dfn","content":"pub fn dfn(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.dfn\nlustre/element/html dfn","url":"lustre/element/html.html#dfn"},{"doc":"lustre/element/html","title":"dialog","content":"pub fn dialog(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.dialog\nlustre/element/html dialog","url":"lustre/element/html.html#dialog"},{"doc":"lustre/element/html","title":"div","content":"pub fn div(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.div\nlustre/element/html div","url":"lustre/element/html.html#div"},{"doc":"lustre/element/html","title":"dl","content":"pub fn dl(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.dl\nlustre/element/html dl","url":"lustre/element/html.html#dl"},{"doc":"lustre/element/html","title":"dt","content":"pub fn dt(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.dt\nlustre/element/html dt","url":"lustre/element/html.html#dt"},{"doc":"lustre/element/html","title":"em","content":"pub fn em(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.em\nlustre/element/html em","url":"lustre/element/html.html#em"},{"doc":"lustre/element/html","title":"embed","content":"pub fn embed(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.embed\nlustre/element/html embed","url":"lustre/element/html.html#embed"},{"doc":"lustre/element/html","title":"fieldset","content":"pub fn fieldset(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.fieldset\nlustre/element/html fieldset","url":"lustre/element/html.html#fieldset"},{"doc":"lustre/element/html","title":"figcaption","content":"pub fn figcaption(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.figcaption\nlustre/element/html figcaption","url":"lustre/element/html.html#figcaption"},{"doc":"lustre/element/html","title":"figure","content":"pub fn figure(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.figure\nlustre/element/html figure","url":"lustre/element/html.html#figure"},{"doc":"lustre/element/html","title":"footer","content":"pub fn footer(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.footer\nlustre/element/html footer","url":"lustre/element/html.html#footer"},{"doc":"lustre/element/html","title":"form","content":"pub fn form(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.form\nlustre/element/html form","url":"lustre/element/html.html#form"},{"doc":"lustre/element/html","title":"h1","content":"pub fn h1(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h1\nlustre/element/html h1","url":"lustre/element/html.html#h1"},{"doc":"lustre/element/html","title":"h2","content":"pub fn h2(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h2\nlustre/element/html h2","url":"lustre/element/html.html#h2"},{"doc":"lustre/element/html","title":"h3","content":"pub fn h3(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h3\nlustre/element/html h3","url":"lustre/element/html.html#h3"},{"doc":"lustre/element/html","title":"h4","content":"pub fn h4(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h4\nlustre/element/html h4","url":"lustre/element/html.html#h4"},{"doc":"lustre/element/html","title":"h5","content":"pub fn h5(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h5\nlustre/element/html h5","url":"lustre/element/html.html#h5"},{"doc":"lustre/element/html","title":"h6","content":"pub fn h6(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.h6\nlustre/element/html h6","url":"lustre/element/html.html#h6"},{"doc":"lustre/element/html","title":"head","content":"pub fn head(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.head\nlustre/element/html head","url":"lustre/element/html.html#head"},{"doc":"lustre/element/html","title":"header","content":"pub fn header(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.header\nlustre/element/html header","url":"lustre/element/html.html#header"},{"doc":"lustre/element/html","title":"hgroup","content":"pub fn hgroup(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.hgroup\nlustre/element/html hgroup","url":"lustre/element/html.html#hgroup"},{"doc":"lustre/element/html","title":"hr","content":"pub fn hr(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.hr\nlustre/element/html hr","url":"lustre/element/html.html#hr"},{"doc":"lustre/element/html","title":"html","content":"pub fn html(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.html\nlustre/element/html html","url":"lustre/element/html.html#html"},{"doc":"lustre/element/html","title":"i","content":"pub fn i(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.i\nlustre/element/html i","url":"lustre/element/html.html#i"},{"doc":"lustre/element/html","title":"iframe","content":"pub fn iframe(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.iframe\nlustre/element/html iframe","url":"lustre/element/html.html#iframe"},{"doc":"lustre/element/html","title":"img","content":"pub fn img(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.img\nlustre/element/html img","url":"lustre/element/html.html#img"},{"doc":"lustre/element/html","title":"input","content":"pub fn input(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.input\nlustre/element/html input","url":"lustre/element/html.html#input"},{"doc":"lustre/element/html","title":"ins","content":"pub fn ins(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.ins\nlustre/element/html ins","url":"lustre/element/html.html#ins"},{"doc":"lustre/element/html","title":"kbd","content":"pub fn kbd(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.kbd\nlustre/element/html kbd","url":"lustre/element/html.html#kbd"},{"doc":"lustre/element/html","title":"label","content":"pub fn label(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.label\nlustre/element/html label","url":"lustre/element/html.html#label"},{"doc":"lustre/element/html","title":"legend","content":"pub fn legend(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.legend\nlustre/element/html legend","url":"lustre/element/html.html#legend"},{"doc":"lustre/element/html","title":"li","content":"pub fn li(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.li\nlustre/element/html li","url":"lustre/element/html.html#li"},{"doc":"lustre/element/html","title":"link","content":"pub fn link(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.link\nlustre/element/html link","url":"lustre/element/html.html#link"},{"doc":"lustre/element/html","title":"main","content":"pub fn main(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.main\nlustre/element/html main","url":"lustre/element/html.html#main"},{"doc":"lustre/element/html","title":"map","content":"pub fn map(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n Used with &lt;area&gt; elements to define an image map (a clickable link area).\n\nSynonyms:\nlustre/element/html.map\nlustre/element/html map","url":"lustre/element/html.html#map"},{"doc":"lustre/element/html","title":"mark","content":"pub fn mark(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.mark\nlustre/element/html mark","url":"lustre/element/html.html#mark"},{"doc":"lustre/element/html","title":"math","content":"pub fn math(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.math\nlustre/element/html math","url":"lustre/element/html.html#math"},{"doc":"lustre/element/html","title":"menu","content":"pub fn menu(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.menu\nlustre/element/html menu","url":"lustre/element/html.html#menu"},{"doc":"lustre/element/html","title":"meta","content":"pub fn meta(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.meta\nlustre/element/html meta","url":"lustre/element/html.html#meta"},{"doc":"lustre/element/html","title":"meter","content":"pub fn meter(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.meter\nlustre/element/html meter","url":"lustre/element/html.html#meter"},{"doc":"lustre/element/html","title":"nav","content":"pub fn nav(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.nav\nlustre/element/html nav","url":"lustre/element/html.html#nav"},{"doc":"lustre/element/html","title":"noscript","content":"pub fn noscript(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.noscript\nlustre/element/html noscript","url":"lustre/element/html.html#noscript"},{"doc":"lustre/element/html","title":"object","content":"pub fn object(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.object\nlustre/element/html object","url":"lustre/element/html.html#object"},{"doc":"lustre/element/html","title":"ol","content":"pub fn ol(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.ol\nlustre/element/html ol","url":"lustre/element/html.html#ol"},{"doc":"lustre/element/html","title":"optgroup","content":"pub fn optgroup(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.optgroup\nlustre/element/html optgroup","url":"lustre/element/html.html#optgroup"},{"doc":"lustre/element/html","title":"option","content":"pub fn option(\n  attrs: List(Attribute(a)),\n  label: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.option\nlustre/element/html option","url":"lustre/element/html.html#option"},{"doc":"lustre/element/html","title":"output","content":"pub fn output(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.output\nlustre/element/html output","url":"lustre/element/html.html#output"},{"doc":"lustre/element/html","title":"p","content":"pub fn p(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.p\nlustre/element/html p","url":"lustre/element/html.html#p"},{"doc":"lustre/element/html","title":"picture","content":"pub fn picture(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.picture\nlustre/element/html picture","url":"lustre/element/html.html#picture"},{"doc":"lustre/element/html","title":"portal","content":"pub fn portal(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.portal\nlustre/element/html portal","url":"lustre/element/html.html#portal"},{"doc":"lustre/element/html","title":"pre","content":"pub fn pre(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.pre\nlustre/element/html pre","url":"lustre/element/html.html#pre"},{"doc":"lustre/element/html","title":"progress","content":"pub fn progress(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.progress\nlustre/element/html progress","url":"lustre/element/html.html#progress"},{"doc":"lustre/element/html","title":"q","content":"pub fn q(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.q\nlustre/element/html q","url":"lustre/element/html.html#q"},{"doc":"lustre/element/html","title":"rp","content":"pub fn rp(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.rp\nlustre/element/html rp","url":"lustre/element/html.html#rp"},{"doc":"lustre/element/html","title":"rt","content":"pub fn rt(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.rt\nlustre/element/html rt","url":"lustre/element/html.html#rt"},{"doc":"lustre/element/html","title":"ruby","content":"pub fn ruby(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.ruby\nlustre/element/html ruby","url":"lustre/element/html.html#ruby"},{"doc":"lustre/element/html","title":"s","content":"pub fn s(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.s\nlustre/element/html s","url":"lustre/element/html.html#s"},{"doc":"lustre/element/html","title":"samp","content":"pub fn samp(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.samp\nlustre/element/html samp","url":"lustre/element/html.html#samp"},{"doc":"lustre/element/html","title":"script","content":"pub fn script(\n  attrs: List(Attribute(a)),\n  js: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.script\nlustre/element/html script","url":"lustre/element/html.html#script"},{"doc":"lustre/element/html","title":"search","content":"pub fn search(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.search\nlustre/element/html search","url":"lustre/element/html.html#search"},{"doc":"lustre/element/html","title":"section","content":"pub fn section(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.section\nlustre/element/html section","url":"lustre/element/html.html#section"},{"doc":"lustre/element/html","title":"select","content":"pub fn select(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.select\nlustre/element/html select","url":"lustre/element/html.html#select"},{"doc":"lustre/element/html","title":"slot","content":"pub fn slot(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.slot\nlustre/element/html slot","url":"lustre/element/html.html#slot"},{"doc":"lustre/element/html","title":"small","content":"pub fn small(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.small\nlustre/element/html small","url":"lustre/element/html.html#small"},{"doc":"lustre/element/html","title":"source","content":"pub fn source(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.source\nlustre/element/html source","url":"lustre/element/html.html#source"},{"doc":"lustre/element/html","title":"span","content":"pub fn span(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.span\nlustre/element/html span","url":"lustre/element/html.html#span"},{"doc":"lustre/element/html","title":"strong","content":"pub fn strong(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.strong\nlustre/element/html strong","url":"lustre/element/html.html#strong"},{"doc":"lustre/element/html","title":"style","content":"pub fn style(\n  attrs: List(Attribute(a)),\n  css: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.style\nlustre/element/html style","url":"lustre/element/html.html#style"},{"doc":"lustre/element/html","title":"sub","content":"pub fn sub(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.sub\nlustre/element/html sub","url":"lustre/element/html.html#sub"},{"doc":"lustre/element/html","title":"summary","content":"pub fn summary(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.summary\nlustre/element/html summary","url":"lustre/element/html.html#summary"},{"doc":"lustre/element/html","title":"sup","content":"pub fn sup(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.sup\nlustre/element/html sup","url":"lustre/element/html.html#sup"},{"doc":"lustre/element/html","title":"svg","content":"pub fn svg(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.svg\nlustre/element/html svg","url":"lustre/element/html.html#svg"},{"doc":"lustre/element/html","title":"table","content":"pub fn table(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.table\nlustre/element/html table","url":"lustre/element/html.html#table"},{"doc":"lustre/element/html","title":"tbody","content":"pub fn tbody(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.tbody\nlustre/element/html tbody","url":"lustre/element/html.html#tbody"},{"doc":"lustre/element/html","title":"td","content":"pub fn td(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.td\nlustre/element/html td","url":"lustre/element/html.html#td"},{"doc":"lustre/element/html","title":"template","content":"pub fn template(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.template\nlustre/element/html template","url":"lustre/element/html.html#template"},{"doc":"lustre/element/html","title":"text","content":"pub fn text(content: String) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.text\nlustre/element/html text","url":"lustre/element/html.html#text"},{"doc":"lustre/element/html","title":"textarea","content":"pub fn textarea(\n  attrs: List(Attribute(a)),\n  content: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.textarea\nlustre/element/html textarea","url":"lustre/element/html.html#textarea"},{"doc":"lustre/element/html","title":"tfoot","content":"pub fn tfoot(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.tfoot\nlustre/element/html tfoot","url":"lustre/element/html.html#tfoot"},{"doc":"lustre/element/html","title":"th","content":"pub fn th(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.th\nlustre/element/html th","url":"lustre/element/html.html#th"},{"doc":"lustre/element/html","title":"thead","content":"pub fn thead(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.thead\nlustre/element/html thead","url":"lustre/element/html.html#thead"},{"doc":"lustre/element/html","title":"time","content":"pub fn time(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.time\nlustre/element/html time","url":"lustre/element/html.html#time"},{"doc":"lustre/element/html","title":"title","content":"pub fn title(\n  attrs: List(Attribute(a)),\n  content: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.title\nlustre/element/html title","url":"lustre/element/html.html#title"},{"doc":"lustre/element/html","title":"tr","content":"pub fn tr(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.tr\nlustre/element/html tr","url":"lustre/element/html.html#tr"},{"doc":"lustre/element/html","title":"track","content":"pub fn track(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.track\nlustre/element/html track","url":"lustre/element/html.html#track"},{"doc":"lustre/element/html","title":"u","content":"pub fn u(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.u\nlustre/element/html u","url":"lustre/element/html.html#u"},{"doc":"lustre/element/html","title":"ul","content":"pub fn ul(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.ul\nlustre/element/html ul","url":"lustre/element/html.html#ul"},{"doc":"lustre/element/html","title":"var","content":"pub fn var(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.var\nlustre/element/html var","url":"lustre/element/html.html#var"},{"doc":"lustre/element/html","title":"video","content":"pub fn video(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.video\nlustre/element/html video","url":"lustre/element/html.html#video"},{"doc":"lustre/element/html","title":"wbr","content":"pub fn wbr(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/html.wbr\nlustre/element/html wbr","url":"lustre/element/html.html#wbr"},{"doc":"lustre/element/html","title":"lustre/element/html","content":"","url":"lustre/element/html.html"},{"doc":"lustre/element/svg","title":"a","content":"pub fn a(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.a\nlustre/element/svg a","url":"lustre/element/svg.html#a"},{"doc":"lustre/element/svg","title":"animate","content":"pub fn animate(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.animate\nlustre/element/svg animate","url":"lustre/element/svg.html#animate"},{"doc":"lustre/element/svg","title":"animate_motion","content":"pub fn animate_motion(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.animate_motion\nlustre/element/svg animate_motion","url":"lustre/element/svg.html#animate_motion"},{"doc":"lustre/element/svg","title":"animate_transform","content":"pub fn animate_transform(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.animate_transform\nlustre/element/svg animate_transform","url":"lustre/element/svg.html#animate_transform"},{"doc":"lustre/element/svg","title":"circle","content":"pub fn circle(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.circle\nlustre/element/svg circle","url":"lustre/element/svg.html#circle"},{"doc":"lustre/element/svg","title":"clip_path","content":"pub fn clip_path(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.clip_path\nlustre/element/svg clip_path","url":"lustre/element/svg.html#clip_path"},{"doc":"lustre/element/svg","title":"defs","content":"pub fn defs(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.defs\nlustre/element/svg defs","url":"lustre/element/svg.html#defs"},{"doc":"lustre/element/svg","title":"desc","content":"pub fn desc(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.desc\nlustre/element/svg desc","url":"lustre/element/svg.html#desc"},{"doc":"lustre/element/svg","title":"ellipse","content":"pub fn ellipse(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.ellipse\nlustre/element/svg ellipse","url":"lustre/element/svg.html#ellipse"},{"doc":"lustre/element/svg","title":"fe_blend","content":"pub fn fe_blend(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_blend\nlustre/element/svg fe_blend","url":"lustre/element/svg.html#fe_blend"},{"doc":"lustre/element/svg","title":"fe_color_matrix","content":"pub fn fe_color_matrix(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_color_matrix\nlustre/element/svg fe_color_matrix","url":"lustre/element/svg.html#fe_color_matrix"},{"doc":"lustre/element/svg","title":"fe_component_transfer","content":"pub fn fe_component_transfer(\n  attrs: List(Attribute(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_component_transfer\nlustre/element/svg fe_component_transfer","url":"lustre/element/svg.html#fe_component_transfer"},{"doc":"lustre/element/svg","title":"fe_composite","content":"pub fn fe_composite(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_composite\nlustre/element/svg fe_composite","url":"lustre/element/svg.html#fe_composite"},{"doc":"lustre/element/svg","title":"fe_convolve_matrix","content":"pub fn fe_convolve_matrix(\n  attrs: List(Attribute(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_convolve_matrix\nlustre/element/svg fe_convolve_matrix","url":"lustre/element/svg.html#fe_convolve_matrix"},{"doc":"lustre/element/svg","title":"fe_diffuse_lighting","content":"pub fn fe_diffuse_lighting(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_diffuse_lighting\nlustre/element/svg fe_diffuse_lighting","url":"lustre/element/svg.html#fe_diffuse_lighting"},{"doc":"lustre/element/svg","title":"fe_displacement_map","content":"pub fn fe_displacement_map(\n  attrs: List(Attribute(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_displacement_map\nlustre/element/svg fe_displacement_map","url":"lustre/element/svg.html#fe_displacement_map"},{"doc":"lustre/element/svg","title":"fe_distant_light","content":"pub fn fe_distant_light(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_distant_light\nlustre/element/svg fe_distant_light","url":"lustre/element/svg.html#fe_distant_light"},{"doc":"lustre/element/svg","title":"fe_drop_shadow","content":"pub fn fe_drop_shadow(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_drop_shadow\nlustre/element/svg fe_drop_shadow","url":"lustre/element/svg.html#fe_drop_shadow"},{"doc":"lustre/element/svg","title":"fe_flood","content":"pub fn fe_flood(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_flood\nlustre/element/svg fe_flood","url":"lustre/element/svg.html#fe_flood"},{"doc":"lustre/element/svg","title":"fe_func_a","content":"pub fn fe_func_a(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_func_a\nlustre/element/svg fe_func_a","url":"lustre/element/svg.html#fe_func_a"},{"doc":"lustre/element/svg","title":"fe_func_b","content":"pub fn fe_func_b(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_func_b\nlustre/element/svg fe_func_b","url":"lustre/element/svg.html#fe_func_b"},{"doc":"lustre/element/svg","title":"fe_func_g","content":"pub fn fe_func_g(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_func_g\nlustre/element/svg fe_func_g","url":"lustre/element/svg.html#fe_func_g"},{"doc":"lustre/element/svg","title":"fe_func_r","content":"pub fn fe_func_r(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_func_r\nlustre/element/svg fe_func_r","url":"lustre/element/svg.html#fe_func_r"},{"doc":"lustre/element/svg","title":"fe_gaussian_blur","content":"pub fn fe_gaussian_blur(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_gaussian_blur\nlustre/element/svg fe_gaussian_blur","url":"lustre/element/svg.html#fe_gaussian_blur"},{"doc":"lustre/element/svg","title":"fe_image","content":"pub fn fe_image(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_image\nlustre/element/svg fe_image","url":"lustre/element/svg.html#fe_image"},{"doc":"lustre/element/svg","title":"fe_merge","content":"pub fn fe_merge(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_merge\nlustre/element/svg fe_merge","url":"lustre/element/svg.html#fe_merge"},{"doc":"lustre/element/svg","title":"fe_merge_node","content":"pub fn fe_merge_node(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_merge_node\nlustre/element/svg fe_merge_node","url":"lustre/element/svg.html#fe_merge_node"},{"doc":"lustre/element/svg","title":"fe_morphology","content":"pub fn fe_morphology(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_morphology\nlustre/element/svg fe_morphology","url":"lustre/element/svg.html#fe_morphology"},{"doc":"lustre/element/svg","title":"fe_offset","content":"pub fn fe_offset(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_offset\nlustre/element/svg fe_offset","url":"lustre/element/svg.html#fe_offset"},{"doc":"lustre/element/svg","title":"fe_point_light","content":"pub fn fe_point_light(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_point_light\nlustre/element/svg fe_point_light","url":"lustre/element/svg.html#fe_point_light"},{"doc":"lustre/element/svg","title":"fe_specular_lighting","content":"pub fn fe_specular_lighting(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_specular_lighting\nlustre/element/svg fe_specular_lighting","url":"lustre/element/svg.html#fe_specular_lighting"},{"doc":"lustre/element/svg","title":"fe_spot_light","content":"pub fn fe_spot_light(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_spot_light\nlustre/element/svg fe_spot_light","url":"lustre/element/svg.html#fe_spot_light"},{"doc":"lustre/element/svg","title":"fe_tile","content":"pub fn fe_tile(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_tile\nlustre/element/svg fe_tile","url":"lustre/element/svg.html#fe_tile"},{"doc":"lustre/element/svg","title":"fe_turbulence","content":"pub fn fe_turbulence(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.fe_turbulence\nlustre/element/svg fe_turbulence","url":"lustre/element/svg.html#fe_turbulence"},{"doc":"lustre/element/svg","title":"foreign_object","content":"pub fn foreign_object(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.foreign_object\nlustre/element/svg foreign_object","url":"lustre/element/svg.html#foreign_object"},{"doc":"lustre/element/svg","title":"g","content":"pub fn g(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.g\nlustre/element/svg g","url":"lustre/element/svg.html#g"},{"doc":"lustre/element/svg","title":"image","content":"pub fn image(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.image\nlustre/element/svg image","url":"lustre/element/svg.html#image"},{"doc":"lustre/element/svg","title":"line","content":"pub fn line(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.line\nlustre/element/svg line","url":"lustre/element/svg.html#line"},{"doc":"lustre/element/svg","title":"linear_gradient","content":"pub fn linear_gradient(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.linear_gradient\nlustre/element/svg linear_gradient","url":"lustre/element/svg.html#linear_gradient"},{"doc":"lustre/element/svg","title":"marker","content":"pub fn marker(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.marker\nlustre/element/svg marker","url":"lustre/element/svg.html#marker"},{"doc":"lustre/element/svg","title":"mask","content":"pub fn mask(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.mask\nlustre/element/svg mask","url":"lustre/element/svg.html#mask"},{"doc":"lustre/element/svg","title":"metadata","content":"pub fn metadata(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.metadata\nlustre/element/svg metadata","url":"lustre/element/svg.html#metadata"},{"doc":"lustre/element/svg","title":"missing_glyph","content":"pub fn missing_glyph(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.missing_glyph\nlustre/element/svg missing_glyph","url":"lustre/element/svg.html#missing_glyph"},{"doc":"lustre/element/svg","title":"mpath","content":"pub fn mpath(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.mpath\nlustre/element/svg mpath","url":"lustre/element/svg.html#mpath"},{"doc":"lustre/element/svg","title":"path","content":"pub fn path(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.path\nlustre/element/svg path","url":"lustre/element/svg.html#path"},{"doc":"lustre/element/svg","title":"pattern","content":"pub fn pattern(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.pattern\nlustre/element/svg pattern","url":"lustre/element/svg.html#pattern"},{"doc":"lustre/element/svg","title":"polygon","content":"pub fn polygon(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.polygon\nlustre/element/svg polygon","url":"lustre/element/svg.html#polygon"},{"doc":"lustre/element/svg","title":"polyline","content":"pub fn polyline(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.polyline\nlustre/element/svg polyline","url":"lustre/element/svg.html#polyline"},{"doc":"lustre/element/svg","title":"radial_gradient","content":"pub fn radial_gradient(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.radial_gradient\nlustre/element/svg radial_gradient","url":"lustre/element/svg.html#radial_gradient"},{"doc":"lustre/element/svg","title":"rect","content":"pub fn rect(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.rect\nlustre/element/svg rect","url":"lustre/element/svg.html#rect"},{"doc":"lustre/element/svg","title":"script","content":"pub fn script(\n  attrs: List(Attribute(a)),\n  js: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.script\nlustre/element/svg script","url":"lustre/element/svg.html#script"},{"doc":"lustre/element/svg","title":"set","content":"pub fn set(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.set\nlustre/element/svg set","url":"lustre/element/svg.html#set"},{"doc":"lustre/element/svg","title":"stop","content":"pub fn stop(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.stop\nlustre/element/svg stop","url":"lustre/element/svg.html#stop"},{"doc":"lustre/element/svg","title":"style","content":"pub fn style(\n  attrs: List(Attribute(a)),\n  css: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.style\nlustre/element/svg style","url":"lustre/element/svg.html#style"},{"doc":"lustre/element/svg","title":"svg","content":"pub fn svg(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.svg\nlustre/element/svg svg","url":"lustre/element/svg.html#svg"},{"doc":"lustre/element/svg","title":"switch","content":"pub fn switch(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.switch\nlustre/element/svg switch","url":"lustre/element/svg.html#switch"},{"doc":"lustre/element/svg","title":"symbol","content":"pub fn symbol(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.symbol\nlustre/element/svg symbol","url":"lustre/element/svg.html#symbol"},{"doc":"lustre/element/svg","title":"text","content":"pub fn text(\n  attrs: List(Attribute(a)),\n  content: String,\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.text\nlustre/element/svg text","url":"lustre/element/svg.html#text"},{"doc":"lustre/element/svg","title":"text_path","content":"pub fn text_path(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.text_path\nlustre/element/svg text_path","url":"lustre/element/svg.html#text_path"},{"doc":"lustre/element/svg","title":"title","content":"pub fn title(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.title\nlustre/element/svg title","url":"lustre/element/svg.html#title"},{"doc":"lustre/element/svg","title":"tspan","content":"pub fn tspan(\n  attrs: List(Attribute(a)),\n  children: List(Element(a)),\n) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.tspan\nlustre/element/svg tspan","url":"lustre/element/svg.html#tspan"},{"doc":"lustre/element/svg","title":"use_","content":"pub fn use_(attrs: List(Attribute(a))) -&gt; Element(a)\n\nSynonyms:\nlustre/element/svg.use_\nlustre/element/svg use_","url":"lustre/element/svg.html#use_"},{"doc":"lustre/element/svg","title":"lustre/element/svg","content":"","url":"lustre/element/svg.html"},{"doc":"lustre/event","title":"checked","content":"pub fn checked(event: Dynamic) -&gt; Result(Bool, List(DecodeError))\n Similar to [`value`](#value), decoding a checkbox&#39;s `checked` state is common\n enough to warrant a dedicated decoder. This attempts to decode\n `event.target.checked` as a boolean.\n\nSynonyms:\nlustre/event.checked\nlustre/event checked","url":"lustre/event.html#checked"},{"doc":"lustre/event","title":"emit","content":"pub fn emit(event: String, data: Json) -&gt; Effect(a)\n Dispatches a custom message from a Lustre component. This lets components\n communicate with their parents the same way native DOM elements do.\n\n Any JSON-serialisable payload can be attached as additional data for any\n event listeners to decode. This data will be on the event&#39;s `detail` property.\n\nSynonyms:\nlustre/event.emit\nlustre/event emit","url":"lustre/event.html#emit"},{"doc":"lustre/event","title":"mouse_position","content":"pub fn mouse_position(\n  event: Dynamic,\n) -&gt; Result(#(Float, Float), List(DecodeError))\n Decodes the mouse position from any event that has a `clientX` and `clientY`\n property.\n\nSynonyms:\nlustre/event.mouse_position\nlustre/event mouse_position","url":"lustre/event.html#mouse_position"},{"doc":"lustre/event","title":"on","content":"pub fn on(\n  name: String,\n  handler: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Attribute(a)\n Listens for the given event and applies the handler to the event object. If\n the handler returns an `Ok` the resulting message will be dispatched, otherwise\n the event (and any decoding error) will be ignored.\n\n The event name is typically an all-lowercase string such as &quot;click&quot; or &quot;mousemove&quot;.\n If you&#39;re listening for non-standard events (like those emitted by a custom\n element) their event names might be slightly different.\n\nSynonyms:\nlustre/event.on\nlustre/event on","url":"lustre/event.html#on"},{"doc":"lustre/event","title":"on_blur","content":"pub fn on_blur(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_blur\nlustre/event on_blur","url":"lustre/event.html#on_blur"},{"doc":"lustre/event","title":"on_check","content":"pub fn on_check(msg: fn(Bool) -&gt; a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_check\nlustre/event on_check","url":"lustre/event.html#on_check"},{"doc":"lustre/event","title":"on_click","content":"pub fn on_click(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_click\nlustre/event on_click","url":"lustre/event.html#on_click"},{"doc":"lustre/event","title":"on_focus","content":"pub fn on_focus(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_focus\nlustre/event on_focus","url":"lustre/event.html#on_focus"},{"doc":"lustre/event","title":"on_input","content":"pub fn on_input(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_input\nlustre/event on_input","url":"lustre/event.html#on_input"},{"doc":"lustre/event","title":"on_keydown","content":"pub fn on_keydown(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key down events on an element, and dispatches a message with the\n current key being pressed.\n\nSynonyms:\nlustre/event.on_keydown\nlustre/event on_keydown","url":"lustre/event.html#on_keydown"},{"doc":"lustre/event","title":"on_keypress","content":"pub fn on_keypress(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key presses on an element, and dispatches a message with the\n current key being pressed.\n\nSynonyms:\nlustre/event.on_keypress\nlustre/event on_keypress","url":"lustre/event.html#on_keypress"},{"doc":"lustre/event","title":"on_keyup","content":"pub fn on_keyup(msg: fn(String) -&gt; a) -&gt; Attribute(a)\n Listens for key up events on an element, and dispatches a message with the\n current key being released.\n\nSynonyms:\nlustre/event.on_keyup\nlustre/event on_keyup","url":"lustre/event.html#on_keyup"},{"doc":"lustre/event","title":"on_mouse_down","content":"pub fn on_mouse_down(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_down\nlustre/event on_mouse_down","url":"lustre/event.html#on_mouse_down"},{"doc":"lustre/event","title":"on_mouse_enter","content":"pub fn on_mouse_enter(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_enter\nlustre/event on_mouse_enter","url":"lustre/event.html#on_mouse_enter"},{"doc":"lustre/event","title":"on_mouse_leave","content":"pub fn on_mouse_leave(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_leave\nlustre/event on_mouse_leave","url":"lustre/event.html#on_mouse_leave"},{"doc":"lustre/event","title":"on_mouse_out","content":"pub fn on_mouse_out(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_out\nlustre/event on_mouse_out","url":"lustre/event.html#on_mouse_out"},{"doc":"lustre/event","title":"on_mouse_over","content":"pub fn on_mouse_over(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_over\nlustre/event on_mouse_over","url":"lustre/event.html#on_mouse_over"},{"doc":"lustre/event","title":"on_mouse_up","content":"pub fn on_mouse_up(msg: a) -&gt; Attribute(a)\n\nSynonyms:\nlustre/event.on_mouse_up\nlustre/event on_mouse_up","url":"lustre/event.html#on_mouse_up"},{"doc":"lustre/event","title":"on_submit","content":"pub fn on_submit(msg: a) -&gt; Attribute(a)\n Listens for the form&#39;s `submit` event, and dispatches the given message. This\n will automatically call [`prevent_default`](#prevent_default) to stop the form\n from submitting.\n\nSynonyms:\nlustre/event.on_submit\nlustre/event on_submit","url":"lustre/event.html#on_submit"},{"doc":"lustre/event","title":"prevent_default","content":"pub fn prevent_default(event: Dynamic) -&gt; Nil\n Calls an event&#39;s `preventDefault` method. If the `Dynamic` does not have a\n `preventDefault` method, this function does nothing.\n\n As the name implies, `preventDefault` will prevent any default action associated\n with an event from occuring. For example, if you call `preventDefault` on a\n `submit` event, the form will not be submitted.\n\n See: https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault\n\nSynonyms:\nlustre/event.prevent_default\nlustre/event prevent_default","url":"lustre/event.html#prevent_default"},{"doc":"lustre/event","title":"stop_propagation","content":"pub fn stop_propagation(event: Dynamic) -&gt; Nil\n Calls an event&#39;s `stopPropagation` method. If the `Dynamic` does not have a\n `stopPropagation` method, this function does nothing.\n\n Stopping event propagation means the event will not &quot;bubble&quot; up to parent\n elements. If any elements higher up in the DOM have event listeners for the\n same event, they will not be called.\n\n See: https://developer.mozilla.org/en-US/docs/Web/API/Event/stopPropagation\n\nSynonyms:\nlustre/event.stop_propagation\nlustre/event stop_propagation","url":"lustre/event.html#stop_propagation"},{"doc":"lustre/event","title":"value","content":"pub fn value(event: Dynamic) -&gt; Result(String, List(DecodeError))\n Decoding an input element&#39;s `value` is such a common operation that we have\n a dedicated decoder for it. This attempts to decoder `event.target.value` as\n a string.\n\nSynonyms:\nlustre/event.value\nlustre/event value","url":"lustre/event.html#value"},{"doc":"lustre/event","title":"lustre/event","content":"","url":"lustre/event.html"},{"doc":"lustre/server_component","title":"component","content":"pub fn component(attrs: List(Attribute(a))) -&gt; Element(a)\n Render the Lustre Server Component client runtime. The content of your server\n component will be rendered inside this element.\n\n **Note**: you must include the `lustre-server-component.mjs` script found in\n the `priv/` directory of the Lustre package in your project&#39;s HTML or using\n the [`script`](#script) function.\n\nSynonyms:\nlustre/server_component.component\nlustre/server_component component","url":"lustre/server_component.html#component"},{"doc":"lustre/server_component","title":"data","content":"pub fn data(json: Json) -&gt; Attribute(a)\n Ocassionally you may want to attach custom data to an event sent to the server.\n This could be used to include a hash of the current build to detect if the\n event was sent from a stale client.\n\n Your event decoders can access this data by decoding `data` property of the\n event object.\n\nSynonyms:\nlustre/server_component.data\nlustre/server_component data","url":"lustre/server_component.html#data"},{"doc":"lustre/server_component","title":"decode_action","content":"pub fn decode_action(\n  dyn: Dynamic,\n) -&gt; Result(Action(a, ServerComponent), List(DecodeError))\n The server component client runtime sends JSON encoded actions for the server\n runtime to execute. Because your own WebSocket server sits between the two\n parts of the runtime, you need to decode these actions and pass them to the\n server runtime yourself.\n\nSynonyms:\nlustre/server_component.decode_action\nlustre/server_component decode_action","url":"lustre/server_component.html#decode_action"},{"doc":"lustre/server_component","title":"emit","content":"pub fn emit(event: String, data: Json) -&gt; Effect(a)\n Instruct any connected clients to emit a DOM event with the given name and\n data. This lets your server component communicate to frontend the same way\n any other HTML elements do: you might emit a `&quot;change&quot;` event when some part\n of the server component&#39;s state changes, for example.\n\n This is a real DOM event and any JavaScript on the page can attach an event\n listener to the server component element and listen for these events.\n\nSynonyms:\nlustre/server_component.emit\nlustre/server_component emit","url":"lustre/server_component.html#emit"},{"doc":"lustre/server_component","title":"encode_patch","content":"pub fn encode_patch(patch: Patch(a)) -&gt; Json\n Encode a DOM patch as JSON you can send to the client runtime to apply. Whenever\n the server runtime re-renders, all subscribed clients will receive a patch\n message they must forward to the client runtime.\n\nSynonyms:\nlustre/server_component.encode_patch\nlustre/server_component encode_patch","url":"lustre/server_component.html#encode_patch"},{"doc":"lustre/server_component","title":"include","content":"pub fn include(properties: List(String)) -&gt; Attribute(a)\n Properties of a JavaScript event object are typically not serialisable. This\n means if we want to pass them to the server we need to copy them into a new\n object first.\n\n This attribute tells Lustre what properties to include. Properties can come\n from nested objects by using dot notation. For example, you could include the\n `id` of the target `element` by passing `[&quot;target.id&quot;]`.\n\n \n import gleam/dynamic\n import gleam/result.{try}\n import lustre/element.{type Element}\n import lustre/element/html\n import lustre/event\n import lustre/server\n\n pub fn custom_button(on_click: fn(String) -&gt; msg) -&gt; Element(msg) {\n   let handler = fn(event) {\n     use target &lt;- try(dynamic.field(&quot;target&quot;, dynamic.dynamic)(event))\n     use id &lt;- try(dynamic.field(&quot;id&quot;, dynamic.string)(target))\n\n     Ok(on_click(id))\n   }\n\n   html.button([event.on_click(handler), server.include([&quot;target.id&quot;])], [\n     element.text(&quot;Click me!&quot;)\n   ])\n }\n \n\nSynonyms:\nlustre/server_component.include\nlustre/server_component include","url":"lustre/server_component.html#include"},{"doc":"lustre/server_component","title":"route","content":"pub fn route(path: String) -&gt; Attribute(a)\n The `route` attribute tells the client runtime what route it should use to\n set up the WebSocket connection to the server. Whenever this attribute is\n changed (by a clientside Lustre app, for example), the client runtime will\n destroy the current connection and set up a new one.\n\nSynonyms:\nlustre/server_component.route\nlustre/server_component route","url":"lustre/server_component.html#route"},{"doc":"lustre/server_component","title":"script","content":"pub fn script() -&gt; Element(a)\n Inline the Lustre Server Component client runtime as a script tag.\n\nSynonyms:\nlustre/server_component.script\nlustre/server_component script","url":"lustre/server_component.html#script"},{"doc":"lustre/server_component","title":"select","content":"pub fn select(\n  sel: fn(fn(a) -&gt; Nil, Subject(b)) -&gt; Selector(a),\n) -&gt; Effect(a)\n On the Erlang target, Lustre&#39;s server component runtime is an OTP\n [actor](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html) that can be\n communicated with using the standard process API and the `Subject` returned\n when starting the server component.\n\n Sometimes, you might want to hand a different `Subject` to a process to restrict\n the type of messages it can send or to distinguish messages from different\n sources from one another. The `select` effect creates a fresh `Subject` each\n time it is run. By returning a `Selector` you can teach the Lustre server\n component runtime how to listen to messages from this `Subject`.\n\n The `select` effect also gives you the dispatch function passed to `effect.from`.\n This is useful in case you want to store the provided `Subject` in your model\n for later use. For example you may subscribe to a pubsub service and later use\n that same `Subject` to unsubscribe.\n\n **Note**: This effect does nothing on the JavaScript runtime, where `Subjects`\n and `Selectors` don&#39;t exist, and is the equivalent of returning `effect.none()`.\n\nSynonyms:\nlustre/server_component.select\nlustre/server_component select","url":"lustre/server_component.html#select"},{"doc":"lustre/server_component","title":"set_selector","content":"pub fn set_selector(: Selector(Action(a, b))) -&gt; Effect(b)\n\n\nSynonyms:\nlustre/server_component.set_selector\nlustre/server_component set_selector","url":"lustre/server_component.html#set_selector"},{"doc":"lustre/server_component","title":"subscribe","content":"pub fn subscribe(\n  id: String,\n  renderer: fn(Patch(a)) -&gt; Nil,\n) -&gt; Action(a, ServerComponent)\n A server component broadcasts patches to be applied to the DOM to any connected\n clients. This action is used to add a new client to a running server component.\n\nSynonyms:\nlustre/server_component.subscribe\nlustre/server_component subscribe","url":"lustre/server_component.html#subscribe"},{"doc":"lustre/server_component","title":"unsubscribe","content":"pub fn unsubscribe(id: String) -&gt; Action(a, ServerComponent)\n Remove a registered renderer from a server component. If no renderer with the\n given id is found, this action has no effect.\n\nSynonyms:\nlustre/server_component.unsubscribe\nlustre/server_component unsubscribe","url":"lustre/server_component.html#unsubscribe"},{"doc":"lustre/server_component","title":"lustre/server_component","content":" &gt; **Note**: server components are currently only supported on the **erlang**\n &gt; target. If it&#39;s important to you that they work on the javascript target,\n &gt; [open an issue](https://github.com/lustre-labs/lustre/issues/new) and tell\n &gt; us why it&#39;s important to you!\n\n Server components are an advanced feature that allows you to run entire\n Lustre applications on the server. DOM changes are broadcasted to a small\n client runtime and browser events are sent back to the server.\n\n ```text\n -- SERVER -----------------------------------------------------------------\n\n                  Msg                            Element(Msg)\n +--------+        v        +----------------+        v        +------+\n |        | &lt;-------------- |                | &lt;-------------- |      |\n | update |                 | Lustre runtime |                 | view |\n |        | --------------&gt; |                | --------------&gt; |      |\n +--------+        ^        +----------------+        ^        +------+\n         #(model, Effect(msg))  |        ^          Model\n                                |        |\n                                |        |\n                    DOM patches |        | DOM events\n                                |        |\n                                v        |\n                        +-----------------------+\n                        |                       |\n                        | Your WebSocket server |\n                        |                       |\n                        +-----------------------+\n                                |        ^\n                                |        |\n                    DOM patches |        | DOM events\n                                |        |\n                                v        |\n -- BROWSER ----------------------------------------------------------------\n                                |        ^\n                                |        |\n                    DOM patches |        | DOM events\n                                |        |\n                                v        |\n                            +----------------+\n                            |                |\n                            | Client runtime |\n                            |                |\n                            +----------------+\n ```\n\n **Note**: Lustre&#39;s server component runtime is separate from your application&#39;s\n WebSocket server. You&#39;re free to bring your own stack, connect multiple\n clients to the same Lustre instance, or keep the application alive even when\n no clients are connected.\n\n Lustre server components run next to the rest of your backend code, your\n services, your database, etc. Real-time applications like chat services, games,\n or components that can benefit from direct access to your backend services\n like an admin dashboard or data table are excellent candidates for server\n components.\n\n ## Examples\n\n Server components are a new feature in Lustre and we&#39;re still working on the\n best ways to use them and show them off. For now, you can find a simple\n undocumented example in the `examples/` directory:\n\n - [`99-server-components`](https://github.com/lustre-labs/lustre/tree/main/examples/99-server-components)\n\n ## Getting help\n\n If you&#39;re having trouble with Lustre or not sure what the right way to do\n something is, the best place to get help is the [Gleam Discord server](https://discord.gg/Fm8Pwmy).\n You could also open an issue on the [Lustre GitHub repository](https://github.com/lustre-labs/lustre/issues).\n","url":"lustre/server_component.html"}]);